/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */
package com.ibm.premises.ct.utils;

import java.util.ResourceBundle;

import com.ibm.ras.RASITraceEvent;
import com.ibm.rfid.premises.logger.BaseLogger;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Container Tracking RAS logger.
 */
@SuppressWarnings("deprecation")
public class CTLogger extends BaseLogger {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

	private static Logger myCTLogger;
	public static final long TRACE_TYPE_FINE = RASITraceEvent.TYPE_LEVEL1;
	public static final long TRACE_TYPE_FINER = RASITraceEvent.TYPE_LEVEL2;
	public static final long TRACE_TYPE_FINEST = Logger.TRACE_TYPE_DEBUG;
	public static final long TRACE_TYPE_PERF = TRACE_TYPE_FINE;
	
	private CTLogger() {
	}

	/* (non-Javadoc)
	 * @see com.ibm.rfid.premises.logger.BaseLogger#_init()
	 */
	@SuppressWarnings("deprecation")
	protected void _init() {
		com.ibm.websphere.ras.Manager manager = com.ibm.websphere.ras.Manager.getManager();
		myWasMessageLogger = manager.createRASMessageLogger("Premises Server Container Tracking", getClass().getName());
		myWasTraceLogger = manager.createRASTraceLogger("Premises Server Container Tracking", getClass().getName());
		manager.addLoggerToGroup(myWasTraceLogger, "Premises Server Container Tracking");
		myResourceBundle = ResourceBundle.getBundle(Constants.RESOURCE_BUNDLE);
	}

	/**
	 * Returns the singleton CTLogger instance
	 * @return singleton CTLogger instance
	 */
	public static Logger singleton() {
		Logger logger = (Logger) myCTLogger;
		if (logger == null) {
			logger = new CTLogger();
			myCTLogger = logger;
		}
		
		return logger;
	}
	
	public boolean isFineTraceEnabled() {
		return this.isTraceEnabled(TRACE_TYPE_FINE);
	}
	
	public boolean isFinerTraceEnabled() {
		return this.isTraceEnabled(TRACE_TYPE_FINER);
	}
	
	public boolean isFinestTraceEnabled() {
		return this.isTraceEnabled(TRACE_TYPE_FINEST);
	}
	
	public boolean isPerformanceTraceEnabled() {
		return this.isTraceEnabled(TRACE_TYPE_PERF);
	}
	
	public boolean isTraceEnabled(long level) {
		return this.myWasTraceLogger.isLoggable(level);
	}
	
}
