/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.utils;

import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.ejb.EJBHome;
import javax.ejb.EJBLocalHome;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.sql.DataSource;

/**
 * ResourceLocator
 */
public class ResourceLocator {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	private static ResourceLocator mySingleton;
	private Map<String, Object> cache;
	private InitialContext ic;
	
	protected ResourceLocator() throws Exception {
		this.cache = new HashMap<String, Object>();
		this.ic = new InitialContext();
	}
	
	public synchronized static ResourceLocator singleton() throws Exception {
		if (mySingleton == null) {
			mySingleton = new ResourceLocator();
		}
		
		return mySingleton;
	}
	
	public DataSource getDataSource(String name) throws Exception {
		DataSource ds = (DataSource) this.cache.get(name);
		if (ds == null) {
			ds = (DataSource) this.ic.lookup(name);
			this.cache.put(name, ds);
		}
		
		return ds;
	}
	
	public ResourceBundle getResourceBundle(String name) {
		ResourceBundle rb = (ResourceBundle) this.cache.get(name);
		if (rb == null) {
			rb = ResourceBundle.getBundle(name);
			this.cache.put(name, rb);
		}
		
		return rb;		
	}
	
	public EJBHome getEJBHome(String name, Class c) throws Exception {
		EJBHome home = (EJBHome) this.cache.get(name);
		if (home == null) {
			Object o = this.ic.lookup(name);
			home = (EJBHome) PortableRemoteObject.narrow(o, c);
			this.cache.put(name, home);
		}
		
		return home;
	}
	
	public EJBLocalHome getEJBLocalHome(String name) throws Exception {
		EJBLocalHome home = (EJBLocalHome) this.cache.get(name);
		if (home == null) {
			home = (EJBLocalHome) this.ic.lookup(name);
			this.cache.put(name, home);
		}
		
		return home;
	}
	
}
