/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.utils;

import java.security.Principal;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;

import com.ibm.rfid.premises.logger.Logger;

/**
 * UI user roles
 */
public class UserRoles {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String BASE = "ibmctBase";
	public static final String PORTAL = "ibmctPortal";
	public static final String PRINT = "ibmctPrint";
	public static final String REPORT = "ibmctReport";
	public static final String ADMIN = "ibmctAdmin";
	private static ResourceBundle rb;
	
	public boolean isUserRoleBase = false;
	public boolean isUserRolePortal = false;
	public boolean isUserRolePrint = false;
	public boolean isUserRoleReport = false;
	public boolean isUserRoleAdmin = false;
	
	protected UserRoles() {
		rb = ResourceBundle.getBundle(Constants.RESOURCE_BUNDLE);
	}
	
	public UserRoles(HttpServletRequest request) throws Exception {
		this();
		
		this.isUserRoleAdmin = this.isUserRole(request, ADMIN);
		if (this.isUserRoleAdmin) {
			this.isUserRoleBase = true;
			this.isUserRolePortal = true;
			this.isUserRolePrint = true;
			this.isUserRoleReport = true;
		} else {
			this.isUserRolePortal = this.isUserRole(request, PORTAL);
			if (this.isUserRolePortal) {
				this.isUserRoleBase = true;
			}
			
			this.isUserRolePrint = this.isUserRole(request, PRINT);
			if (this.isUserRolePrint) {
				this.isUserRoleBase = true;
			}
			
			this.isUserRoleReport = this.isUserRole(request, REPORT);
			if (this.isUserRoleReport) {
				this.isUserRoleBase = true;
			}
			
			// only check for BASE access if it is not already set
			if (this.isUserRoleBase == false) {
				this.isUserRoleBase = this.isUserRole(request, BASE);
			}
		}
	}
	
	protected boolean isUserRole(HttpServletRequest request, String role) throws Exception {
		Utils utils = Utils.singleton();
		Logger logger = utils.logger;
		logger.traceEntry(this, "isUserRole");
		
		Principal principal = request.getUserPrincipal();
		
		boolean isRole = (principal == null || request.isUserInRole(role));
		if (CTLogger.singleton().isTraceEnabled()) {
			String principalName = null;
			if (principal != null) {
				principalName = principal.getName();
			}

			logger.trace(Logger.TRACE_TYPE_DEBUG, this, "isUserRole", utils.formatMessage(rb.getString("ibmct.userroles.trace.user.principal"), principalName));
			logger.trace(Logger.TRACE_TYPE_DEBUG, this, "isUserRole", utils.formatMessage(rb.getString("ibmct.userroles.trace.user.role"), role, String.valueOf(isRole)));
		}
		
		logger.traceExit(this, "isUserRole");
		
		return isRole;
	}
	
	public String toString() {
		return "UserRoles: " +
			ADMIN + "=" + this.isUserRoleAdmin +
			"; " + PORTAL + "=" + this.isUserRolePortal +
			"; " + PRINT + "=" + this.isUserRolePrint +
			"; " + REPORT + "=" + this.isUserRoleReport +
			"; " + BASE + "=" + this.isUserRoleBase;		
	}

}
