/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.ejb;

import java.lang.reflect.Method;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.TreeMap;

import javax.ejb.CreateException;
import javax.ejb.SessionContext;
import javax.sql.DataSource;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.dto.ColumnMetaData;
import com.ibm.premises.ct.dto.ObserveReport;
import com.ibm.premises.ct.dto.Owner;
import com.ibm.premises.ct.exceptions.JdbcFacadeException;
import com.ibm.premises.ct.utils.Constants;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.rfid.premises.common.utils.DBUtils;
import com.ibm.rfid.premises.logger.Logger;
import com.ibm.ws.rsadapter.jdbc.WSJdbcConnection;
import com.ibm.ws.rsadapter.jdbc.WSJdbcUtil;

/**
 * Container Tracking JDBC session facade.
 */

public class JdbcSessionFacadeBean implements javax.ejb.SessionBean {
	private static final long serialVersionUID = 1L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
		+ "All Rights Reserved * Licensed Materials - Property of IBM "
		+ "US Government Users Restricted Rights - Use, duplication or "
		+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	private static Utils utils;
	private static ResourceBundle rb;
	private static DataSource dataSource;
	private SessionContext mySessionCtx;

	/////////////////////////////////////////////
	//
	// OBSERVE REPORT
	//
	/////////////////////////////////////////////
	/** 
	 * Create a new observe report.
	 * @param asset The {@link com.ibm.premises.ct.dto.ObserveReport report} to create.
	 * @throws JdbcFacadeException 
	 */
	public void createObserveReport(List<ObserveReport> reports) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "createObserveReport");
			
			String sql = 
				"insert into sage.ct_report_observe " +
				"(id, session_id, record_time, event_time, process_action, location_username, supplier_id, tag_urn, tag_description, tag_hex) " +
				"values " +
				"(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
			
			con = dataSource.getConnection();
			con.setAutoCommit(false);
			ps = con.prepareStatement(sql);
			
			for (Iterator<ObserveReport> i=reports.iterator(); i.hasNext(); ) {
				ObserveReport report = i.next();

				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "sql=" + sql);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.id=" + report.getId());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.sessionId=" + report.getSessionId());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.recordTime=" + report.getRecordTime());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.eventTime=" + report.getEventTime());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.processAction=" + report.getProcessAction());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.locationUserName=" + report.getLocationUserName());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.supplierId=" + report.getSupplierId());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.tagUrn=" + report.getTagUrn());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.tagDescription=" + report.getTagDescription());
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", "report.tagHexValue=" + report.getTagRawValue());
				}
				
				ps.setString(1, report.getId());
				ps.setString(2, report.getSessionId());
				ps.setString(3, report.getRecordTime());
				ps.setString(4, report.getEventTime());
				ps.setString(5, report.getProcessAction());
				ps.setString(6, report.getLocationUserName());
				ps.setString(7, report.getSupplierId());
				ps.setString(8, report.getTagUrn());
				ps.setString(9, report.getTagDescription());
				ps.setString(10, report.getTagRawValue());
			
				int count = ps.executeUpdate();
				if (count != 1) {
					// SQL failed!
					String message = rb.getString("ibmct.jdbc.trace.error.create.observe.report.failed");
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReport", message);
					logger.traceExit(this, "createObserveReport");
				}
			}
			
			con.commit();
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "createObserveReport", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "createObserveReport");
		}

	}
	
	/** 
	 * Delete an existing objserve report with the matching sessionId.
	 * @param sessionId The sessionId of the report to delete.
	 * @throws JdbcFacadeException
	 */
	public void deleteObserveReport(String sessionId) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "deleteObserveReport");
			
			String sql = "delete from sage.ct_report_observe where session_id=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteObserveReport", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteObserveReport", "session_id=" + sessionId);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, sessionId);
			
			int count = ps.executeUpdate();
			if (count == 0) {
				// do nothing as there may not be any existing reports to delete
			}
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "deleteObserveReport", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "deleteObserveReport");
		}
	}
	
	
	/////////////////////////////////////////////
	//
	// COLUMN META DATA
	//
	/////////////////////////////////////////////
	public List<ColumnMetaData> queryDBMetaData(String schema, String table) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		
		try {
			logger.traceEntry(this, "queryDBMetaData");
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "queryDBMetaData", "schema=" + schema);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "queryDBMetaData", "table=" + table);
			}
			
			con = dataSource.getConnection();
			DatabaseMetaData dmd = con.getMetaData();
			ResultSet rs = dmd.getColumns(null, schema, table, null);

			Map<String, ColumnMetaData> columns = new TreeMap<String, ColumnMetaData>();
			
			ColumnMetaData cmd = null;
			
			while (rs.next()) {
				String columnName = rs.getString("COLUMN_NAME");
				int dataType = rs.getInt("DATA_TYPE");
				int columnSize = rs.getInt("COLUMN_SIZE");
				boolean nullable = (DatabaseMetaData.columnNullable == rs.getInt("NULLABLE"));
				
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "queryDBMetaData", "columnName=" + columnName);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "queryDBMetaData", "dataType=" + dataType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "queryDBMetaData", "columnSize=" + columnSize);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "queryDBMetaData", "nullable=" + nullable);
				}
				
				cmd = new ColumnMetaData(columnName);
				cmd.setDataType(dataType);
				cmd.setSize(columnSize);
				cmd.setNullable(nullable);
				
				columns.put(columnName, cmd);
			}
			
			rs = dmd.getPrimaryKeys(null, schema, table);
			while (rs.next()) {
				String columnName = rs.getString("COLUMN_NAME");
				cmd = columns.get(columnName);
				cmd.setPrimaryKey(true);
			}
			
			return new ArrayList<ColumnMetaData>(columns.values());
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "queryDBMetaData", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "queryDBMetaData");
		}
	}
	
	/////////////////////////////////////////////
	//
	// ASSET
	//
	/////////////////////////////////////////////
	
	/** 
	 * Create a new asset.
	 * @param asset The {@link com.ibm.premises.ct.dto.Asset asset} to create.
	 * @throws JdbcFacadeException 
	 */
	public void createAsset(Asset asset) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "createAsset");
			
			String sql = "insert into sage.ct_asset (id, description, epc_asset_type, epc_pack_type, label_data, is_hidden) values (?, ?, ?, ?, ?, ?)";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", "asset.id=" + asset.getId());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", "asset.description=" + asset.getDescription());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", "asset.epcAssetType=" + asset.getEpcAssetType());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", "asset.epcPackType=" + asset.getEpcPackType());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", "asset.labelData=" + asset.getLabelDataAsXML());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", "asset.hidden=" + asset.isHidden());
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, asset.getId());
			ps.setString(2, asset.getDescription());
			ps.setInt(3, asset.getEpcAssetType());
			ps.setString(4, asset.getEpcPackType());
			
			if (DBUtils.isDB2(con)) {
				// DB2
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", rb.getString("ibmct.jdbc.trace.info.database.db2"));
				}
				
				ps.setString(5, asset.getLabelDataAsXML());
			} else {
				// Oracle
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", rb.getString("ibmct.jdbc.trace.info.database.oracle"));
				}
				
				Connection nativeConnection = (Connection) WSJdbcUtil.getNativeConnection((WSJdbcConnection) con);
			    
				// use reflection to dynamically create the Oracle XMLType
				// XMLType xmlType = XMLType.createXML(nativeConnection, xml);
				Class [] parmTypes = {Connection.class, String.class};
				Method method = Class.forName("oracle.xdb.XMLType").getMethod("createXML", parmTypes);
				Object [] parms = {nativeConnection, asset.getLabelDataAsXML()};
				Object xmlTypeObj = method.invoke(null, parms);
				
			    ps.setObject(5, xmlTypeObj);
			}
			
			ps.setString(6, String.valueOf(asset.isHidden()));
			
			int count = ps.executeUpdate();
			if (count != 1) {
				// SQL failed!
				String message = rb.getString("ibmct.jdbc.trace.error.create.asset.failed");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createAsset", message);
				logger.traceExit(this, "createAsset");
				throw new JdbcFacadeException(message);
			}
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "createAsset", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "createAsset");
		}

	}
	
	/** 
	 * Delete an existing asset.
	 * @param id The ID of the asset to delete.
	 * @throws JdbcFacadeException
	 */
	public void deleteAsset(String id) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "deleteAsset");
			
			String sql = "delete from sage.ct_asset where id=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteAsset", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteAsset", "id=" + id);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, id);
			
			int count = ps.executeUpdate();
			if (count != 1) {
				// SQL failed!
				String message = rb.getString("ibmct.jdbc.trace.error.delete.asset.failed");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteAsset", message);
				logger.traceExit(this, "deleteAsset");
				throw new JdbcFacadeException(message);
			}
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "deleteAsset", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "deleteAsset");
		}
	}
	
	/** 
	 * Update an existing asset.
	 * @param asset The {@link com.ibm.premises.ct.dto.Asset asset} to update.
	 * @throws JdbcFacadeException
	 */
	public void updateAsset(Asset asset) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "updateAsset");
			
			String sql = "update sage.ct_asset set description=?, epc_asset_type=?, epc_pack_type=?, label_data=?, is_hidden=? where id=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", "asset.id=" + asset.getId());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", "asset.description=" + asset.getDescription());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", "asset.epcAssetType=" + asset.getEpcAssetType());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", "asset.epcPackType=" + asset.getEpcPackType());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", "asset.printTitle=" + asset.getLabelDataAsXML());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", "asset.hidden=" + asset.isHidden());
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, asset.getDescription());
			ps.setInt(2, asset.getEpcAssetType());
			ps.setString(3, asset.getEpcPackType());
			
			if (DBUtils.isDB2(con)) {
				// DB2
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", rb.getString("ibmct.jdbc.trace.info.database.db2"));
				}
				
				ps.setString(4, asset.getLabelDataAsXML());
			} else {
				// Oracle
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", rb.getString("ibmct.jdbc.trace.info.database.oracle"));
				}
				
				Connection nativeConnection = (Connection) WSJdbcUtil.getNativeConnection((WSJdbcConnection) con);
			    
				// use reflection to dynamically create the Oracle XMLType
				// XMLType xmlType = XMLType.createXML(nativeConnection, xml);
				Class [] parmTypes = {Connection.class, String.class};
				Method method = Class.forName("oracle.xdb.XMLType").getMethod("createXML", parmTypes);
				Object [] parms = {nativeConnection, asset.getLabelDataAsXML()};
				Object xmlTypeObj = method.invoke(null, parms);
				
			    ps.setObject(4, xmlTypeObj);
			}
			
			ps.setString(5, String.valueOf(asset.isHidden()));
			ps.setString(6, asset.getId());
			
			int count = ps.executeUpdate();
			if (count != 1) {
				// SQL failed!
				String message = rb.getString("ibmct.jdbc.trace.error.update.asset.failed");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateAsset", message);
				logger.traceExit(this, "updateAsset");
				throw new JdbcFacadeException(message);
			}
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "updateAsset", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "updateAsset");
		}
	}
	
	/** 
	 * Find an asset by its ID.
	 * @param id The asset ID to find.
	 * @return {@link com.ibm.premises.ct.dto.Asset asset}
	 * @throws JdbcFacadeException
	 */
	public Asset findAsset(String id) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "findAsset");
			
			String sql = "select * from sage.ct_asset where id=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "id=" + id);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, id);
			
			Asset asset = null;
			
			ResultSet rs = ps.executeQuery();
			while (rs.next()) {
				String description = rs.getString("description");
				int epcAssetType = rs.getInt("epc_asset_type");
				String epcPackType = rs.getString("epc_pack_type");
				boolean hidden = String.valueOf(true).equals(rs.getString("is_hidden"));
				String labelDataXML = null;
				
				if (DBUtils.isDB2(con)) {
					// DB2
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", rb.getString("ibmct.jdbc.trace.info.database.db2"));
					}
					
					labelDataXML = rs.getString("label_data");
				} else {
					// Oracle
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", rb.getString("ibmct.jdbc.trace.info.database.oracle"));
					}
		
					Class opaqueClass = Class.forName("oracle.sql.OPAQUE");
					Class xmlTypeClass = Class.forName("oracle.xdb.XMLType");
					Class [] parmTypes = {opaqueClass};
					Method createXMLMethod = xmlTypeClass.getMethod("createXML", parmTypes);
					Object [] parms = {rs.getObject("label_data")};
					Object xmlTypeObj = createXMLMethod.invoke(null, parms);
					Method getColbValMethod = xmlTypeClass.getMethod("getClobVal", (Class[]) null);
					Clob clob = (Clob) getColbValMethod.invoke(xmlTypeObj, (Object[]) null);
					long len = clob.length();					
					labelDataXML  = clob.getSubString(1, (int) len);
					if (labelDataXML == null) {
						labelDataXML = "";
					}
				}
				
				asset = new Asset(id);
				asset.setDescription(description);
				asset.setEpcAssetType(epcAssetType);
				asset.setEpcPackType(epcPackType);
				asset.setLabelDataFromXML(labelDataXML);
				asset.setHidden(hidden);
				
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "Asset found");
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "id=" + id);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "description=" + description);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "epcAssetType=" + epcAssetType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "epcPackType=" + epcPackType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "labelData=" + labelDataXML);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", "hidden=" + hidden);
				}
				
				break;
			}
			
			if (asset == null) {
				String message = utils.formatMessage(rb.getString("ibmct.jdbc.trace.error.find.asset.failed"), id);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAsset", message);
				throw new JdbcFacadeException(message);
			}
			
			return asset;
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "findAsset", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "findAsset");
		}

	}
	
	/** 
	 * Find all assets.
 	 * @return java.util.Collection<{@link com.ibm.premises.ct.dto.Asset asset> sorted by asset description
	 * @throws JdbcFacadeException
	 */
	public List<Asset> findAllAssets() throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "findAllAssets");
			
			String sql = "select * from sage.ct_asset";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "sql=" + sql);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			
			Map<String, Asset> assets = new TreeMap<String, Asset>();
			Asset asset = null;
			
			ResultSet rs = ps.executeQuery();
			while (rs.next()) {
				String id = rs.getString("id");
				String description = rs.getString("description");
				int epcAssetType = rs.getInt("epc_asset_type");
				String epcPackType = rs.getString("epc_pack_type");
				boolean hidden = String.valueOf(true).equals(rs.getString("is_hidden"));
				String labelDataXML = null;
				
				if (DBUtils.isDB2(con)) {
					// DB2
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", rb.getString("ibmct.jdbc.trace.info.database.db2"));
					}
					
					labelDataXML = rs.getString("label_data");
				} else {
					// Oracle
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", rb.getString("ibmct.jdbc.trace.info.database.oracle"));
					}
		
					Class opaqueClass = Class.forName("oracle.sql.OPAQUE");
					Class xmlTypeClass = Class.forName("oracle.xdb.XMLType");
					Class [] parmTypes = {opaqueClass};
					Method createXMLMethod = xmlTypeClass.getMethod("createXML", parmTypes);
					Object [] parms = {rs.getObject("label_data")};
					Object xmlTypeObj = createXMLMethod.invoke(null, parms);
					Method getColbValMethod = xmlTypeClass.getMethod("getClobVal", (Class[]) null);
					Clob clob = (Clob) getColbValMethod.invoke(xmlTypeObj, (Object[]) null);
					long len = clob.length();					
					labelDataXML  = clob.getSubString(1, (int) len);
					if (labelDataXML == null) {
						labelDataXML = "";
					}
				}
				
				asset = new Asset(id);
				asset.setDescription(description);
				asset.setEpcAssetType(epcAssetType);
				asset.setEpcPackType(epcPackType);
				asset.setLabelDataFromXML(labelDataXML);
				asset.setHidden(hidden);
				
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "Asset found");
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "id=" + id);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "description=" + description);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "epcAssetType=" + epcAssetType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "epcPackType=" + epcPackType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "labelData=" + labelDataXML);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssets", "hidden=" + hidden);
				}
				
				// sort the assets by description
				assets.put(asset.getDescription(), asset);
			}
			
			return new ArrayList<Asset>(assets.values());
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "findAllAssets", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "findAllAssets");
		}

	}
	
	/** 
	 * Find all assets with the given epcAssetType.
	 * @param epcAssetType The epcAssetType to find.
 	 * @return java.util.Collection<{@link com.ibm.premises.ct.dto.Asset asset> sorted by asset description
	 * @throws JdbcFacadeException
	 */
	public List<Asset> findAllAssetsForEpcAssetType(int epcAssetType) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "findAllAssetsForEpcAssetType");
			
			String sql = "select * from sage.ct_asset where epc_asset_type=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "epc_asset_type=" + epcAssetType);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setInt(1, epcAssetType);
			
			Map<String, Asset> assets = new TreeMap<String, Asset>();
			Asset asset = null;
			
			ResultSet rs = ps.executeQuery();
			while (rs.next()) {
				String id = rs.getString("id");
				String description = rs.getString("description");
				String epcPackType = rs.getString("epc_pack_type");
				boolean hidden = String.valueOf(true).equals(rs.getString("is_hidden"));
				String labelDataXML = null;
				
				if (DBUtils.isDB2(con)) {
					// DB2
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", rb.getString("ibmct.jdbc.trace.info.database.db2"));
					}
					
					labelDataXML = rs.getString("label_data");
				} else {
					// Oracle
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", rb.getString("ibmct.jdbc.trace.info.database.oracle"));
					}
		
					Class opaqueClass = Class.forName("oracle.sql.OPAQUE");
					Class xmlTypeClass = Class.forName("oracle.xdb.XMLType");
					Class [] parmTypes = {opaqueClass};
					Method createXMLMethod = xmlTypeClass.getMethod("createXML", parmTypes);
					Object [] parms = {rs.getObject("label_data")};
					Object xmlTypeObj = createXMLMethod.invoke(null, parms);
					Method getColbValMethod = xmlTypeClass.getMethod("getClobVal", (Class[]) null);
					Clob clob = (Clob) getColbValMethod.invoke(xmlTypeObj, (Object[]) null);
					long len = clob.length();					
					labelDataXML  = clob.getSubString(1, (int) len);
					if (labelDataXML == null) {
						labelDataXML = "";
					}
				}
				
				asset = new Asset(id);
				asset.setDescription(description);
				asset.setEpcAssetType(epcAssetType);
				asset.setEpcPackType(epcPackType);
				asset.setLabelDataFromXML(labelDataXML);
				asset.setHidden(hidden);
				
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "Asset found");
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "id=" + id);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "description=" + description);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "epcAssetType=" + epcAssetType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "epcPackType=" + epcPackType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "labelData=" + labelDataXML);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllAssetsForEpcAssetType", "hidden=" + hidden);
				}
				
				// sort the assets by description
				assets.put(asset.getDescription(), asset);
			}
			
			return new ArrayList<Asset>(assets.values());
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "findAllAssetsForEpcAssetType", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "findAllAssetsForEpcAssetType");
		}

	}
	
	/////////////////////////////////////////////
	//
	// OWNER
	//
	/////////////////////////////////////////////
	
	/** 
	 * Create a new owner.
	 * @param asset The {@link com.ibm.premises.ct.dto.Owner owner} to create.
	 * @throws JdbcFacadeException
	 */
	public void createOwner(Owner owner) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "createOwner");
			
			String sql = "insert into sage.ct_owner (id, description) values (?, ?)";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createOwner", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createOwner", "owner.id=" + owner.getId());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createOwner", "owner.description=" + owner.getDescription());
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, owner.getId());
			ps.setString(2, owner.getDescription());
			
			int count = ps.executeUpdate();
			if (count != 1) {
				// SQL failed!
				String message = rb.getString("ibmct.jdbc.trace.error.create.owner.failed");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createOwner", message);
				logger.traceExit(this, "createOwner");
				throw new JdbcFacadeException(message);
			}
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "createOwner", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "createOwner");
		}
	}
	
	/** 
	 * Delete an existing owner.
	 * @param id The ID of the owner to delete.
	 * @throws JdbcFacadeException
	 */
	public void deleteOwner(String id) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "deleteOwner");
			
			String sql = "delete from sage.ct_owner where id=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteOwner", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteOwner", "id=" + id);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, id);
			
			int count = ps.executeUpdate();
			if (count != 1) {
				// SQL failed!
				String message = rb.getString("ibmct.jdbc.trace.error.delete.owner.failed");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "deleteOwner", message);
				logger.traceExit(this, "deleteOwner");
				throw new JdbcFacadeException(message);
			}
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "deleteOwner", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "deleteOwner");
		}
	}
	
	/** 
	 * Update an existing owner.
	 * @param asset The {@link com.ibm.premises.ct.dto.Owner owner} to update.
	 * @throws JdbcFacadeException
	 */
	public void updateOwner(Owner owner) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "updateOwner");
			
			String sql = "update sage.ct_owner set description=? where id=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateOwner", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateOwner", "owner.id=" + owner.getId());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateOwner", "owner.description=" + owner.getDescription());
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, owner.getDescription());
			ps.setString(2, owner.getId());
			
			int count = ps.executeUpdate();
			if (count != 1) {
				// SQL failed!
				String message = rb.getString("ibmct.jdbc.trace.error.update.owner.failed");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "updateOwner", message);
				logger.traceExit(this, "updateOwner");
				throw new JdbcFacadeException(message);
			}
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "updateOwner", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "updateOwner");
		}
	}
	
	/** 
	 * Find an owner by its ID.
	 * @param id The owner ID to find.
	 * @return {@link com.ibm.premises.ct.dto.Owner owner}
	 * @throws JdbcFacadeException
	 */
	public Owner findOwner(String id) throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "findOwner");
			
			String sql = "select * from sage.ct_owner where id=?";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findOwner", "sql=" + sql);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findOwner", "id=" + id);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			ps.setString(1, id);
			
			Owner owner = null;
			
			ResultSet rs = ps.executeQuery();
			while (rs.next()) {
				String description = rs.getString("description");
				
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findOwner", "Owner found");
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findOwner", "id=" + id);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findOwner", "description=" + description);
				}
				
				owner = new Owner(id);
				owner.setDescription(description);
				
				break;
			}
			
			if (owner == null) {
				String message = utils.formatMessage(rb.getString("ibmct.jdbc.trace.error.find.owner.failed"), id);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findOwner", message);
				throw new JdbcFacadeException(message);
			}
			
			return owner;
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "findOwner", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "findOwner");
		}

	}
	
	/** 
	 * Find all owners.
 	 * @return java.util.Collection<{@link com.ibm.premises.ct.dto.Asset asset> sorted by asset description
	 * @throws JdbcFacadeException
	 */
	public List<Owner> findAllOwners() throws JdbcFacadeException {
		Logger logger = utils.logger;
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "findAllOwners");
			
			String sql = "select * from sage.ct_owner";
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllOwners", "sql=" + sql);
			}
			
			con = dataSource.getConnection();
			
			ps = con.prepareStatement(sql);
			
			Map<String, Owner> owners = new TreeMap<String, Owner>();
			Owner owner = null;
			
			ResultSet rs = ps.executeQuery();
			while (rs.next()) {
				String id = rs.getString("id");
				String description = rs.getString("description");
				
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllOwners", "Owner found");
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllOwners", "id=" + id);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findAllOwners", "description=" + description);
				}
				
				owner = new Owner(id);
				owner.setDescription(description);
				
				// sort the owners by description
				owners.put(owner.getDescription(), owner);
			}
			
			return new ArrayList<Owner>(owners.values());
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "findAllOwners", e);
			e.printStackTrace();
			throw new JdbcFacadeException(e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "findAllOwners");
		}

	}
	
	/////////////////////////////////////////////
	//
	// EJB lifecycle methods
	//
	/////////////////////////////////////////////
	
	/**
	 * getSessionContext
	 */
	public SessionContext getSessionContext() {
		return mySessionCtx;
	}
	
	/**
	 * setSessionContext
	 */
	public void setSessionContext(SessionContext ctx) {
		mySessionCtx = ctx;
	}
	
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws CreateException {
		utils = Utils.singleton();
		rb = ResourceBundle.getBundle("com.ibm.premises.ct.ejb.resources.JdbcMessages");
		
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "ejbCreate");
		
			dataSource = utils.locator.getDataSource(Constants.JDBC_REF);
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "ejbCreate", e);
			e.printStackTrace();
			throw new CreateException(e.getMessage());
		} finally {
			logger.traceExit(this, "ejbCreate");
		}
	}
	
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
	
}
