/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.dto;

import java.io.Serializable;
import java.io.StringReader;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * DTO for ASSET table.
 */
public class Asset implements Serializable, Comparable {
	private static final long serialVersionUID = 1L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	private String id;
	private String description;
	private int epcAssetType;
	private String epcPackType;
	private Map<String, String> labelData;
	private boolean hidden;

	private static final String LABEL_NODE = "labelData";
	private static final String VARIABLE_NODE = "variable";
	private static final String NAME_ATTR = "name";
	private static final String VALUE_ATTR = "value";
	private static final String LABEL_DATA_XML = "<" + LABEL_NODE + ">{0}</" + LABEL_NODE + ">";
	private static final String LABEL_DATA_VAR_XML = "<" + VARIABLE_NODE + " " + NAME_ATTR + "=\"{0}\" " + VALUE_ATTR + "=\"{1}\"/>";
	
	public Asset() {
		this.labelData = new HashMap<String, String>();
		this.hidden = false;
	}
	
	public Asset(String id) {
		this();
		this.setId(id);
	}
	
	public Map<String, String> getLabelData() {
		return this.labelData;
	}
	
	public void setLabelData(Map<String, String> labelData) {
		this.labelData = labelData;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public int getEpcAssetType() {
		return epcAssetType;
	}

	public void setEpcAssetType(int epcAssetType) {
		this.epcAssetType = epcAssetType;
	}

	public String getEpcPackType() {
		return epcPackType;
	}

	public void setEpcPackType(String epcPackType) {
		this.epcPackType = epcPackType;
	}

	public String getId() {
		return id;
	}

	protected void setId(String id) {
		this.id = id;
	}
	
	public boolean isHidden() {
		return hidden;
	}

	public void setHidden(boolean hidden) {
		this.hidden = hidden;
	}

	public String getLabelDataAsXML() {
		if (this.labelData == null) {
			return "";
		}
		
		StringBuffer variables = new StringBuffer();
		
		MessageFormat formatter = new MessageFormat(LABEL_DATA_VAR_XML);
		
		for (Iterator i=this.labelData.entrySet().iterator(); i.hasNext(); ) {
			Map.Entry me = (Map.Entry) i.next();
			String key = (String) me.getKey();
			String value = (String) me.getValue();
			String variableXML = formatter.format(new Object[] {key, value});
			variables.append(variableXML);
		}
		
		String xml = MessageFormat.format(LABEL_DATA_XML, new Object[] {variables.toString()});
		return xml;
	}
	
	@SuppressWarnings("unchecked")
	public void setLabelDataFromXML(String xml) throws Exception {
		if (xml == null) {
			return;
		}
		
		Map labelData = new HashMap<String, String>();
		
		DOMParser parser = new DOMParser();
		parser.parse(new InputSource(new StringReader(xml)));
		
		Document document = parser.getDocument();
		
		NodeList nodes = document.getElementsByTagName(VARIABLE_NODE);
		
		for (int i=0; i<nodes.getLength(); i++) {
			Element element = (Element) nodes.item(i);
			String name = element.getAttribute(NAME_ATTR);
			String value = element.getAttribute(VALUE_ATTR);
			labelData.put(name, value);
		}
		
		this.setLabelData(labelData);
	}
	
	public String toString() {
		return this.description;
	}

	/**
	 * Sorts assets by ascending description
	 */
	public int compareTo(Object o) {
		Asset asset = (Asset) o;
		if (this.description.compareTo(asset.getDescription()) < 0) {
			return -1;
		} else if (this.description.compareTo(asset.getDescription()) > 0) {
			return 1;
		} else {
			return 0;
		}
	}
	
	public boolean equals(Object o) {
		Asset asset = null;
		try {
			asset = (Asset) o;
		} catch (Exception e) {
			return false;
		}

		boolean isDescription = false;
		try {
			isDescription = this.description.equals(asset.getDescription());
		} catch (Exception e) {
			if (this.description == null && asset.getDescription() == null) {
				isDescription = true;
			} else {
				isDescription = false;
			}
		}
		
		boolean isEpcAssetType = false;
		try {
			isEpcAssetType = this.epcAssetType == asset.getEpcAssetType();
		} catch (Exception e) {
			isEpcAssetType = false;
		}
		
		boolean isEpcPackType = false;
		try {
			isEpcPackType = this.epcPackType.equals(asset.getEpcPackType());
		} catch (Exception e) {
			if (this.epcPackType == null && asset.getEpcPackType() == null) {
				isEpcPackType = true;
			} else {
				isEpcPackType = false;
			}
		}
		
		boolean isId = false;
		try {
			isId = this.id.equals(asset.getId());
		} catch (Exception e) {
			if (this.id == null && asset.getId() == null) {
				isId = true;
			} else {
				isId = false;
			}
		}
		
		boolean isLabelData = false;
		try {
			isLabelData = this.labelData.equals(asset.getLabelData());
		} catch (Exception e) {
			if (this.labelData == null && asset.getLabelData() == null) {
				isLabelData = true;
			} else {
				isLabelData = false;
			}
		}
		
		return
			isDescription &&
			isEpcAssetType &&
			isEpcPackType &&
			isId &&
			isLabelData;
	}
	
	public int hashCode() {
		int result = 0;
		
		try {
			result += this.description.hashCode();
		} catch (Exception e) {
			// no-op
		}

		result += this.epcAssetType;

		try {
			result += this.epcPackType.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		try {
			result += this.id.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		try {
			result += this.labelData.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		return result;			
	}
}
