/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.dto;

import java.io.Serializable;

/**
 * DTO for table column metadata.
 */
public class ColumnMetaData implements Serializable, Comparable {
	private static final long serialVersionUID = 1L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	private String name;
	private int dataType;
	private int size;
	private boolean nullable;
	private boolean primaryKey;
	
	public ColumnMetaData() {
	}
	
	public ColumnMetaData(String name) {
		this();
		this.setName(name);
	}
		
	public int getDataType() {
		return dataType;
	}

	public void setDataType(int dataType) {
		this.dataType = dataType;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public boolean isNullable() {
		return nullable;
	}

	public void setNullable(boolean nullable) {
		this.nullable = nullable;
	}

	public boolean isPrimaryKey() {
		return primaryKey;
	}

	public void setPrimaryKey(boolean primaryKey) {
		this.primaryKey = primaryKey;
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public String toString() {
		return this.name;
	}

	/**
	 * Sorts by ascending name
	 */
	public int compareTo(Object o) {
		ColumnMetaData asset = (ColumnMetaData) o;
		if (this.name.compareTo(asset.getName()) < 0) {
			return -1;
		} else if (this.name.compareTo(asset.getName()) > 0) {
			return 1;
		} else {
			return 0;
		}
	}
	
	public boolean equals(Object o) {
		ColumnMetaData cmd = null;
		try {
			cmd = (ColumnMetaData) o;
		} catch (Exception e) {
			return false;
		}

		boolean isName = false;
		try {
			isName = this.name.equals(cmd.getName());
		} catch (Exception e) {
			if (this.name == null && cmd.getName() == null) {
				isName = true;
			} else {
				isName = false;
			}
		}
		
		boolean isDataType = false;
		try {
			isDataType = this.dataType == cmd.getDataType();
		} catch (Exception e) {
			isDataType = false;
		}
		
		boolean isSize = false;
		try {
			isSize = this.size == cmd.getSize();
		} catch (Exception e) {
			isSize = false;
		}
		
		boolean isNullable = false;
		try {
			isNullable = this.nullable == cmd.isNullable();
		} catch (Exception e) {
			isNullable = false;
		}
		
		boolean isPrimaryKey = false;
		try {
			isPrimaryKey = this.primaryKey == cmd.isPrimaryKey();
		} catch (Exception e) {
			isPrimaryKey = false;
		}
		
		return
			isName &&
			isDataType &&
			isSize &&
			isNullable &&
			isPrimaryKey;
	}
	
	public int hashCode() {
		int result = 0;
		
		try {
			result += this.name.hashCode();
		} catch (Exception e) {
			// no-op
		}

		result += this.dataType;
		
		result += this.size;

		result += this.nullable ? 1 : 0;

		result += this.primaryKey ? 1 : 0;
		
		return result;			
	}
}
