/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.dto;

import java.io.Serializable;

/**
 * DTO for OWNER table.
 */
public class Owner implements Serializable, Comparable {
	private static final long serialVersionUID = 1L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	private String id;
	private String description;
	
	public Owner() {
	}
	
	public Owner(String id) {
		this.setId(id);
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getId() {
		return id;
	}

	protected void setId(String id) {
		this.id = id;
	}
	
	public String toString() {
		return this.description;
	}
	
	/**
	 * Sorts owners by ascending description
	 */
	public int compareTo(Object o) {
		Owner owner = (Owner) o;
		if (this.description.compareTo(owner.getDescription()) < 0) {
			return -1;
		} else if (this.description.compareTo(owner.getDescription()) > 0) {
			return 1;
		} else {
			return 0;
		}
	}
	
	public boolean equals(Object o) {
		Owner owner = null;
		try {
			owner = (Owner) o;
		} catch (Exception e) {
			return false;
		}

		boolean isDescription = false;
		try {
			isDescription = this.description.equals(owner.getDescription());
		} catch (Exception e) {
			if (this.description == null && owner.getDescription() == null) {
				isDescription = true;
			} else {
				isDescription = false;
			}
		}
		
		boolean isId = false;
		try {
			isId = this.id.equals(owner.getId());
		} catch (Exception e) {
			if (this.id == null && owner.getId() == null) {
				isId = true;
			} else {
				isId = false;
			}
		}

		return
			isDescription &&
			isId;
	}
	
	public int hashCode() {
		int result = 0;
		
		try {
			result += this.description.hashCode();
		} catch (Exception e) {
			// no-op
		}

		try {
			result += this.id.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		return result;			
	}
	
}
