/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.dto;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * DTO for Premises location properties.
 */
public class PremisesLocation implements Serializable, Comparable {
	private static final long serialVersionUID = 1L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	private String id;
	private String description;
	private boolean started;
	private Map<String, String> metaData;
	
	protected PremisesLocation() {
	}
	
	public PremisesLocation(String id) {
		this.setId(id);
	}

	public String getId() {
		return this.id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
	
	public String toString() {
		return this.description;
	}
	
	public boolean isStarted() {
		return started;
	}

	public void setStarted(boolean started) {
		this.started = started;
	}

	public Map<String, String> getMetaData() {
		if (this.metaData == null) {
			return new HashMap<String, String>();
		}
		
		return metaData;
	}

	public void setMetaData(Map<String, String> metaData) {
		this.metaData = metaData;
	}

	/**
	 * Sorts locations by ascending description
	 */
	public int compareTo(Object o) {
		PremisesLocation printer = (PremisesLocation) o;
		if (this.description.compareTo(printer.getDescription()) < 0) {
			return -1;
		} else if (this.description.compareTo(printer.getDescription()) > 0) {
			return 1;
		} else {
			return 0;
		}
	}
	
	public boolean equals(Object o) {
		PremisesLocation location = null;
		try {
			location = (PremisesLocation) o;
		} catch (Exception e) {
			return false;
		}

		boolean isDescription = false;
		try {
			isDescription = this.description.equals(location.getDescription());
		} catch (Exception e) {
			if (this.description == null && location.getDescription() == null) {
				isDescription = true;
			} else {
				isDescription = false;
			}
		}
		
		boolean isId = false;
		try {
			isId = this.id.equals(location.getId());
		} catch (Exception e) {
			if (this.id == null && location.getId() == null) {
				isId = true;
			} else {
				isId = false;
			}
		}
		
		boolean isStarted = false;
		try {
			isStarted = this.started == location.isStarted();
		} catch (Exception e) {
			isStarted = false;
		}
		
		boolean isMetaData = false;
		try {
			isMetaData = this.metaData.equals(location.getMetaData());
		} catch (Exception e) {
			if (this.metaData == null && location.getMetaData() == null) {
				isMetaData = true;
			} else {
				isMetaData = false;
			}
		}

		return
			isDescription &&
			isId &&
			isStarted &&
			isMetaData;
	}
	
	public int hashCode() {
		int result = 0;
		
		try {
			result += this.description.hashCode();
		} catch (Exception e) {
			// no-op
		}

		try {
			result += this.id.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		try {
			result += this.metaData.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		return result;			
	}
	
}
