/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.dto;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * DTO for Premises EPC profile properties.
 */
public class PremisesProfile implements Serializable, Comparable {
	private static final long serialVersionUID = 1L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	private String id;
	private String description;
	private List<String> packTypeNames;
	
	protected PremisesProfile() {
	}
	
	public PremisesProfile(String id, String description) {
		this.setId(id);
		this.setDescription(description);
	}

	public String getId() {
		return this.id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
	
	public void setPackTypeNames(List<String> packTypeNames) {
		this.packTypeNames = packTypeNames;
	}
	
	public List<String> getPackTypeNames() {
		if (this.packTypeNames == null) {
			return new ArrayList<String>();
		} else {
			return this.packTypeNames;
		}
	}
	
	public String toString() {
		return this.description;
	}
	
	/**
	 * Sorts profiles by ascending description
	 */
	public int compareTo(Object o) {
		PremisesProfile printer = (PremisesProfile) o;
		if (this.description.compareTo(printer.getDescription()) < 0) {
			return -1;
		} else if (this.description.compareTo(printer.getDescription()) > 0) {
			return 1;
		} else {
			return 0;
		}
	}
	
	public boolean equals(Object o) {
		PremisesProfile profile = null;
		try {
			profile = (PremisesProfile) o;
		} catch (Exception e) {
			return false;
		}

		boolean isDescription = false;
		try {
			isDescription = this.description.equals(profile.getDescription());
		} catch (Exception e) {
			if (this.description == null && profile.getDescription() == null) {
				isDescription = true;
			} else {
				isDescription = false;
			}
		}
		
		boolean isId = false;
		try {
			isId = this.id.equals(profile.getId());
		} catch (Exception e) {
			if (this.id == null && profile.getId() == null) {
				isId = true;
			} else {
				isId = false;
			}
		}
		
		boolean isPackTypes = false;
		try {
			isPackTypes = this.packTypeNames.equals(profile.getPackTypeNames());
		} catch (Exception e) {
			if (this.packTypeNames == null && profile.getPackTypeNames() == null) {
				isPackTypes = true;
			} else {
				isPackTypes = false;
			}
		}

		return
			isDescription &&
			isId &&
			isPackTypes;
	}
	
	public int hashCode() {
		int result = 0;
		
		try {
			result += this.description.hashCode();
		} catch (Exception e) {
			// no-op
		}

		try {
			result += this.id.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		try {
			result += this.packTypeNames.hashCode();
		} catch (Exception e) {
			// no-op
		}
		
		return result;			
	}
	
}
