package com.ibm.premises.ct.ejb;

import java.util.SortedSet;

import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.dto.PremisesPrinter;
import com.ibm.premises.ct.dto.PremisesProfile;
import com.ibm.premises.ct.exceptions.PremisesFacadeException;

/**
 * Remote interface for Enterprise Bean: PremisesSessionFacade
 */
public interface PremisesSessionFacade extends javax.ejb.EJBObject {

	/**
	 * @param key
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public String getPremisesProperty(String key) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param key
	 * @param defaultValue
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public String getPremisesProperty(String key, String defaultValue) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public SortedSet<PremisesPrinter> getAllPrinters() throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public SortedSet<PremisesProfile> getAllProfiles() throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param id
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public PremisesProfile getProfile(String id) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param id
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public PremisesPrinter getPrinter(String id) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public SortedSet<PremisesLocation> getAllLocations() throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public String getLocationStatus(String locationId) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public void stopLocation(String locationId) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public boolean isLocationStarted(String locationId) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public boolean isLocationStopped(String locationId) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public PremisesLocation getLocation(String locationId) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @param process
	 * @param supplierId
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public void startLocation(String locationId, String process, String supplierId) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param process
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String toEpcisBizStep(String process) throws java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String toEpcisBizLocation(String locationId) throws java.rmi.RemoteException;

	/**
	 * @param bizStep
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String extractProcessFromEpcisBizStep(String bizStep) throws java.rmi.RemoteException;

	/**
	 * @param bizLocation
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String extractLocationFromEpcisBizLocation(String bizLocation) throws java.rmi.RemoteException;

	/**
	 * @param epcUrn
	 * @return
	 * @throws PremisesFacadeException
	 * @throws java.rmi.RemoteException
	 */
	public String toEpcRawURI(String epcUrn) throws PremisesFacadeException, java.rmi.RemoteException;

	/**
	 * @param locationId
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String toEpcisReadPoint(String locationId) throws java.rmi.RemoteException;

	/**
	 * @param readPoint
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String extractLocationFromEpcisReadLocation(String readPoint) throws java.rmi.RemoteException;

	/**
	 * @param process
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String toEpcisDisposition(String process) throws java.rmi.RemoteException;

	/**
	 * @param disposition
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String extractDispositionFromEpcisDisposition(String disposition) throws java.rmi.RemoteException;

	/**
	 * @param shipFromBizLocation
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String extractSupplierFromEpcisShipFromBizLocation(String shipFromBizLocation) throws java.rmi.RemoteException;

	/**
	 * @param shipToBizLocation
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String extractSupplierFromEpcisShipToBizLocation(String shipToBizLocation) throws java.rmi.RemoteException;

	/**
	 * @param supplierId
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String toEpcisShipFromBizLocation(String supplierId) throws java.rmi.RemoteException;

	/**
	 * @param supplierId
	 * @return
	 * @throws java.rmi.RemoteException
	 */
	public String toEpcisShipToBizLocation(String supplierId) throws java.rmi.RemoteException;
}
