package com.ibm.premises.ct.ejb;

import java.util.SortedSet;

import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.dto.PremisesPrinter;
import com.ibm.premises.ct.dto.PremisesProfile;
import com.ibm.premises.ct.exceptions.PremisesFacadeException;

/**
 * Local interface for Enterprise Bean: PremisesSessionFacade
 */
public interface PremisesSessionFacadeLocal extends javax.ejb.EJBLocalObject {

	/**
	 * @param key
	 * @return
	 * @throws PremisesFacadeException
	 */
	public String getPremisesProperty(String key) throws PremisesFacadeException;

	/**
	 * @param key
	 * @param defaultValue
	 * @return
	 * @throws PremisesFacadeException
	 */
	public String getPremisesProperty(String key, String defaultValue) throws PremisesFacadeException;

	/**
	 * @return
	 * @throws PremisesFacadeException
	 */
	public SortedSet<PremisesPrinter> getAllPrinters() throws PremisesFacadeException;

	/**
	 * @return
	 * @throws PremisesFacadeException
	 */
	public SortedSet<PremisesProfile> getAllProfiles() throws PremisesFacadeException;

	/**
	 * @param id
	 * @return
	 * @throws PremisesFacadeException
	 */
	public PremisesProfile getProfile(String id) throws PremisesFacadeException;

	/**
	 * @param id
	 * @return
	 * @throws PremisesFacadeException
	 */
	public PremisesPrinter getPrinter(String id) throws PremisesFacadeException;

	/**
	 * @return
	 * @throws PremisesFacadeException
	 */
	public SortedSet<PremisesLocation> getAllLocations() throws PremisesFacadeException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 */
	public String getLocationStatus(String locationId) throws PremisesFacadeException;

	/**
	 * @param locationId
	 * @throws PremisesFacadeException
	 */
	public void stopLocation(String locationId) throws PremisesFacadeException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 */
	public boolean isLocationStarted(String locationId) throws PremisesFacadeException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 */
	public boolean isLocationStopped(String locationId) throws PremisesFacadeException;

	/**
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 */
	public PremisesLocation getLocation(String locationId) throws PremisesFacadeException;

	/**
	 * @param locationId
	 * @param process
	 * @param supplierId
	 * @throws PremisesFacadeException
	 */
	public void startLocation(String locationId, String process, String supplierId) throws PremisesFacadeException;

	/**
	 * @param process
	 * @return
	 */
	public String toEpcisBizStep(String process);

	/**
	 * @param locationId
	 * @return
	 */
	public String toEpcisBizLocation(String locationId);

	/**
	 * @param bizStep
	 * @return
	 */
	public String extractProcessFromEpcisBizStep(String bizStep);

	/**
	 * @param bizLocation
	 * @return
	 */
	public String extractLocationFromEpcisBizLocation(String bizLocation);

	/**
	 * @param epcUrn
	 * @return
	 * @throws PremisesFacadeException
	 */
	public String toEpcRawURI(String epcUrn) throws PremisesFacadeException;

	/**
	 * @param locationId
	 * @return
	 */
	public String toEpcisReadPoint(String locationId);

	/**
	 * @param readPoint
	 * @return
	 */
	public String extractLocationFromEpcisReadLocation(String readPoint);

	/**
	 * @param process
	 * @return
	 */
	public String toEpcisDisposition(String process);

	/**
	 * @param disposition
	 * @return
	 */
	public String extractDispositionFromEpcisDisposition(String disposition);

	/**
	 * @param shipFromBizLocation
	 * @return
	 */
	public String extractSupplierFromEpcisShipFromBizLocation(String shipFromBizLocation);

	/**
	 * @param shipToBizLocation
	 * @return
	 */
	public String extractSupplierFromEpcisShipToBizLocation(String shipToBizLocation);

	/**
	 * @param supplierId
	 * @return
	 */
	public String toEpcisShipFromBizLocation(String supplierId);

	/**
	 * @param supplierId
	 * @return
	 */
	public String toEpcisShipToBizLocation(String supplierId);
}
