/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.ejb;

import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.ejb.CreateException;
import javax.ejb.SessionContext;

import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.CommissioningRuc;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.EpcRuc;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.ObservationRuc;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.PrintRuc;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.ReportingRuc;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.ValidationRuc;
import com.ibm.premises.ct.exceptions.RucFacadeException;
import com.ibm.premises.ct.utils.CTLogger;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.premises.reusable.commissioning.Commissioning;
import com.ibm.premises.reusable.commissioning.CommissioningHome;
import com.ibm.premises.reusable.epc.EPC;
import com.ibm.premises.reusable.epc.EPCHome;
import com.ibm.premises.reusable.observation.Observation;
import com.ibm.premises.reusable.observation.ObservationHome;
import com.ibm.premises.reusable.printing.Printing;
import com.ibm.premises.reusable.printing.PrintingHome;
import com.ibm.premises.reusable.reporting.Reporting;
import com.ibm.premises.reusable.reporting.ReportingHome;
import com.ibm.premises.reusable.validation.Validation;
import com.ibm.premises.reusable.validation.ValidationHome;
import com.ibm.rfid.premises.logger.Logger;
import com.ibm.sensorevent.model.IBMSensorEvent;
import com.ibm.sensorevent.model.IPayloadMetaData;
import com.ibm.sensorevent.model.ISensorEvent;

/**
 * Container Tracking RUC session facade.
 */
public class RucSessionFacadeBean implements javax.ejb.SessionBean {
	private static final long serialVersionUID = 3206093459760846163L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
		+ "All Rights Reserved * Licensed Materials - Property of IBM "
		+ "US Government Users Restricted Rights - Use, duplication or "
		+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	private static Utils utils;
	private static ResourceBundle rb;
	private SessionContext mySessionCtx;
	
	/////////////////////////////////////////////
	//
	//	 RUC: Validation
	//
	/////////////////////////////////////////////
	public boolean validate(String tagUrn) throws RucFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "validate");
			
			Map<String, String> data = new HashMap<String, String>();
			data.put(ValidationRuc.TAG_URN, tagUrn);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.data"), data.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "validate", message);
			}
			
			ValidationHome home = (ValidationHome) utils.locator.getEJBHome(ValidationRuc.JNDI, ValidationHome.class);
			Validation ruc = home.create();

			boolean isValid = ruc.validateMap(data);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.validate.result"), String.valueOf(isValid));
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "validate", message);
			}

			return isValid;
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "validate", e);
			throw new RucFacadeException(e);
		} finally {
			logger.traceExit(this, "validate");
		}
	}
	
	/////////////////////////////////////////////
	//
	//	 RUC: Observe
	//
	/////////////////////////////////////////////
	public void observe(ISensorEvent event) throws RucFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "observe");
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.data"), event.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "observe", message);
			}
			
			ObservationHome home = (ObservationHome) utils.locator.getEJBHome(ObservationRuc.JNDI, ObservationHome.class);
			Observation ruc = home.create();

			ruc.observeEvent(event);
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "observe", e);
			throw new RucFacadeException(e);
		} finally {
			logger.traceExit(this, "observe");
		}
	}
	
	/////////////////////////////////////////////
	//
	//	 RUC: Commission
	//
	/////////////////////////////////////////////
	public void commission(String eventSourceId, String tagUrn, String assetType) throws RucFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "commission");
			
			Map<String, String> data = new HashMap<String, String>();
			data.put(CommissioningRuc.TAG_URN, tagUrn);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.data"), data.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "commission", message);
			}
			
			ISensorEvent event = IBMSensorEvent.getInstance(CommissioningRuc.EVENT_TYPE, data);
			event.getHeader().setSourceId(eventSourceId);
			
			IPayloadMetaData payloadMetaData = event.getPayloadMetaData();
			payloadMetaData.addStringAttribute(CommissioningRuc.BIZ_STEP_KEY, CommissioningRuc.BIZ_STEP_VALUE);
			payloadMetaData.addStringAttribute(CommissioningRuc.DISPOSITION_KEY, CommissioningRuc.DISPOSITION_VALUE);
			payloadMetaData.addStringAttribute(CommissioningRuc.READ_POINT_KEY, eventSourceId);
			payloadMetaData.addStringAttribute("epcis." + CommissioningRuc.ASSET_TYPE_KEY, assetType != null ? assetType : "");
			
			CommissioningHome home = (CommissioningHome) utils.locator.getEJBHome(CommissioningRuc.JNDI, CommissioningHome.class);
			Commissioning ruc = home.create();
			
			ruc.commissionEvent(event);
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "commission", e);
			throw new RucFacadeException(e);
		} finally {
			logger.traceExit(this, "commission");
		}
	}
	
	/////////////////////////////////////////////
	//
	//	 RUC: EPC
	//
	/////////////////////////////////////////////
	public String epcgen(String epcProfileId, String epcPackType, String epcAssetType) throws RucFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "epcgen");
			
			Map<String, String> data = new HashMap<String, String>();
			data.put(EpcRuc.EPC_PROFILE_ID, epcProfileId);
			data.put(EpcRuc.EPC_PACK_TYPE, epcPackType);
			data.put(EpcRuc.EPC_ASSET_TYPE, epcAssetType);
			data.put(EpcRuc.EPC_TAG_QUANTITY, String.valueOf(EpcRuc.DEFAULT_TAG_QUANTITY));
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.data"), data.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "epcgen", message);
			}
			
			EPCHome home = (EPCHome) utils.locator.getEJBHome(EpcRuc.JNDI, EPCHome.class);
			EPC ruc = home.create();
			
			String[] epcgen = ruc.epcgenMap(data);
			
			// since we're only requesting 1 tag, always look at element 0
			// in the returned array
			String epcValue = null;
			if (epcgen != null && epcgen.length > 0) {
				epcValue = epcgen[0];
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.epcgen.result"), epcValue);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "epcgen", message);
			}
			
			return epcValue;
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "epcgen", e);
			throw new RucFacadeException(e);
		} finally {
			logger.traceExit(this, "epcgen");
		}
	}
	
	public Map epcDecode(String hexEPC) throws RucFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "epcDecode");
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.epcgen.decode.epc"), hexEPC);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "epcDecode", message);
			}
			
			EPCHome home = (EPCHome) utils.locator.getEJBHome(EpcRuc.JNDI, EPCHome.class);
			EPC ruc = home.create();
			
			Map result = ruc.epcdecode(hexEPC);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.epcgen.result"), result.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "epcDecode", message);
			}
			
			return result;
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "epcDecode", e);
			throw new RucFacadeException(e);
		} finally {
			logger.traceExit(this, "epcDecode");
		}
	}
	
	/////////////////////////////////////////////
	//
	//	 RUC: Print
	//
	/////////////////////////////////////////////
	public void print(String mode, String printerId, String epcProfileName, String epcPackType, String epcUrn, Map labelData) throws RucFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "print");
			
			Map<String, Object> data = new HashMap<String, Object>();
			data.put(PrintRuc.MODE, mode);
			data.put(PrintRuc.PRINTER_ID, printerId);
			data.put(PrintRuc.EPC_PROFILE_NAME, epcProfileName);
			data.put(PrintRuc.EPC_URN, epcUrn);
			data.put(PrintRuc.EPC_PACK_TYPE, epcPackType);
			data.put(PrintRuc.LABEL_DATA, labelData);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.data"), data.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "print", message);
			}
			
			PrintingHome home = (PrintingHome) utils.locator.getEJBHome(PrintRuc.JNDI, PrintingHome.class);
			Printing ruc = home.create();
			
			ruc.printMap(data);
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "print", e);
			throw new RucFacadeException(e);
		} finally {
			logger.traceExit(this, "print");
		}
	}

	/////////////////////////////////////////////
	//
	//	 RUC: Reporting
	//
	/////////////////////////////////////////////
	public Map[] getLocationTagHistory(String location, String startDate, String endDate) throws RucFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getLocationTagHistory");
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getLocationTagHistory", "location=" + location);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getLocationTagHistory", "startDate=" + startDate);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getLocationTagHistory", "endDate=" + endDate);
			}
			
			ReportingHome home = (ReportingHome) utils.locator.getEJBHome(ReportingRuc.JNDI, ReportingHome.class);
			Reporting ruc = home.create();
			
			Map[] result = ruc.getLocationTagHistory(location, startDate, endDate);
			
			if (logger.isTraceEnabled()) {
				if (result != null) {
					for (int i=0; i<result.length; i++) {
						String message = utils.formatMessage(rb.getString("ibmct.ruc.trace.info.reporting.result"), result[i].toString());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getLocationTagHistory", message);
					}
				}
			}
			
			return result;
		} catch (Throwable e) {
			e.printStackTrace();
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getLocationTagHistory", e);
			throw new RucFacadeException(e);
		} finally {
			logger.traceExit(this, "getLocationTagHistory");
		}
	}
	
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
		utils = Utils.singleton();
		Logger logger = utils.logger;
		rb = ResourceBundle.getBundle("com.ibm.premises.ct.ejb.resources.RucMessages");
		
		try {
			logger.traceEntry(this, "ejbCreate");
		} catch (Exception e) {
			logger.traceException(CTLogger.MESSAGE_TYPE_ERROR, this, "ejbCreate", e);
			e.printStackTrace();
			throw new CreateException(e.getMessage());
		} finally {
			logger.traceExit(this, "ejbCreate");
		}
	}
	
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
	
}
