/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.ejb;

import com.ibm.premises.ct.utils.Constants;
import com.ibm.premises.reusable.constants.Constants.CommissioningRUCConstants;
import com.ibm.premises.reusable.constants.Constants.EPCRUCConstants;
import com.ibm.premises.reusable.constants.Constants.ObjectEventConstants;
import com.ibm.premises.reusable.constants.Constants.PrintingRUCConstants;

/**
 * RUC facade constants
 */
public class RucSessionFacadeConstants {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static class EpcRuc {
		public static final String JNDI = "ejb/com/ibm/premises/reusable/epc/EPCHome";
		public static final String EPC_PROFILE_ID = EPCRUCConstants.EPC_RUC_PROFILE_NAME;
		public static final String EPC_PACK_TYPE = EPCRUCConstants.EPC_RUC_PACK_TYPE;
		public static final String EPC_ASSET_TYPE = EPCRUCConstants.EPC_RUC_ASSET_TYPE;
		public static final String EPC_TAG_QUANTITY = EPCRUCConstants.EPC_RUC_TAG_QUANTITY;
		public static final int DEFAULT_TAG_QUANTITY = 1;
	}
	
	public static class PrintRuc {
		public static final String JNDI = "ejb/com/ibm/premises/reusable/printing/PrintingHome";
		public static final String PRINTER_ID = PrintingRUCConstants.PRINT_RUC_PRINTER_ID;
		public static final String EPC_PROFILE_NAME = PrintingRUCConstants.PRINT_RUC_PROFILE_NAME;
		public static final String EPC_PACK_TYPE = PrintingRUCConstants.PRINT_RUC_PACK_TYPE;
		public static final String EPC_URN = PrintingRUCConstants.PRINT_RUC_EPC_VALUE;
		public static final String LABEL_DATA = PrintingRUCConstants.PRINT_RUC_LABELDATA;
		public static final String MODE = PrintingRUCConstants.PRINT_RUC_PRINT_MODE_KEY;
		public static final String MODE_PRINT = PrintingRUCConstants.PRINT_RUC_PRINT_MODE;
		public static final String MODE_REPRINT = PrintingRUCConstants.PRINT_RUC_REPRINT_MODE;
	}
	
	public static class ReportingRuc {
		public static final String JNDI = "ejb/com/ibm/premises/reusable/reporting/ReportingHome";
		public static final String RECORD_TIME = ObjectEventConstants.RECORD_TIME_KEY_OUT;
		public static final String EVENT_TIME = ObjectEventConstants.EVENT_TIME_KEY_OUT;
		public static final String BIZ_STEP = ObjectEventConstants.BIZSTEP_KEY_OUT;
		public static final String BIZ_LOCATION = ObjectEventConstants.BIZLOCATION_KEY_OUT;
		public static final String SHIP_TO_BIZ_LOCATION = Constants.EPCISXML_TO_BIZ_LOCATION;
		public static final String SHIP_FROM_BIZ_LOCATION = Constants.EPCISXML_FROM_BIZ_LOCATION;
		public static final String TAG_LIST = ObjectEventConstants.TAGLIST_KEY_INOUT;
		public static final String EPCIS_EXT = Constants.EPCISXML_EXTENSION;
		public static final String EXT_TAG_ID = Constants.EPCISXML_TAG_ID;
		public static final String EXT_DESCRIPTION = Constants.EPCISXML_TAG_DESCRIPTION;
		public static final String EXT_LOCATION_USER_NAME = Constants.EPCISXML_LOCATION_USER_NAME;
	}
	
	public static class ValidationRuc {
		public static final String JNDI = "ejb/com/ibm/premises/reusable/validation/ValidationHome";
		public static final String TAG_LIST = ObjectEventConstants.TAGLIST_KEY_INOUT;
		public static final String TAG_URN = ObjectEventConstants.TAG_KEY_INOUT;
	}
	
	public static class ObservationRuc {
		public static final String JNDI = "ejb/com/ibm/premises/reusable/observation/ObservationHome";
	}

	public static class CommissioningRuc {
		public static final String JNDI = "ejb/com/ibm/premises/reusable/commissioning/CommissioningHome";
		public static final String TAG_LIST = ObjectEventConstants.TAGLIST_KEY_INOUT;
		public static final String TAG_URN = ObjectEventConstants.TAG_KEY_INOUT;
		public static final String EVENT_TYPE = CommissioningRUCConstants.COMM_RUC_EVENTTYPE_VALUE + CommissioningRUCConstants.COMM_RUC_COMM_ACTION_VALUE;
		public static final String DISPOSITION_KEY = ObjectEventConstants.DISPOSITION_KEY_OUT;
		public static final String DISPOSITION_VALUE = Constants.EPCISXML_DISPOSITION_PREFIX + Constants.EPCISXML_DISPOSITION_ACTIVE;
		public static final String BIZ_STEP_KEY = ObjectEventConstants.BIZSTEP_KEY_OUT;
		public static final String BIZ_STEP_VALUE = Constants.EPCISXML_BIZ_STEP_PREFIX + Constants.PROCESS_COMMISSIONING;
		public static final String READ_POINT_KEY = ObjectEventConstants.READPOINT_KEY_OUT;
		public static final String ASSET_TYPE_KEY = Constants.EPCISXML_ASSET_TYPE;
		public static final String EPCIS_EXT_EPCIS = Constants.EPCISXML_EPCIS;
	}
	
}
