/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

var ajaxErrorUnsupportedBrowser = 'Please use a supported Web browser.';

var ajaxXMLHttpFactories = [
	function () {return new XMLHttpRequest()},
	function () {return new ActiveXObject("Msxml2.XMLHTTP")},
	function () {return new ActiveXObject("Msxml3.XMLHTTP")},
	function () {return new ActiveXObject("Microsoft.XMLHTTP")}
];

function ajaxCreateXMLHTTPObject() {
	xmlhttp = false;
	
	for (var i=0; i<ajaxXMLHttpFactories.length; i++) {
		try {
			xmlhttp = ajaxXMLHttpFactories[i]();
		} catch (e) {
			continue;
		}
		break;
	}
	
	if (!xmlhttp) {
		alert(ajaxErrorUnsupportedBrowser);
		throw "ERROR";
	}
	
	return xmlhttp;
}

//
// Synchronous AJAX call (GET) to URL; return the result from the URL (if any)
//
function ajaxAccessUrl(url) {
	// append "?" if the url does not already contain it
	if (url.indexOf("?") == -1) {
		url += "?";
	}
	
	// adding the timestamp to the URL is required in order to access
	// the URL again and not use the cached result
	url += "&timestamp=" + getTimestamp();
	xmlhttp = ajaxCreateXMLHTTPObject();
	xmlhttp.open("GET", url, false);
	xmlhttp.send(null);
	result = trim(xmlhttp.responseText);
	return result;
}

//
// Asynchronous AJAX call (GET) to URL; do not return any result
//
function ajaxAccessAsyncUrl(url) {
	// append "?" if the url does not already contain it
	if (url.indexOf("?") == -1) {
		url += "?";
	}
	
	// adding the timestamp to the URL is required in order to access
	// the URL again and not use the cached result
	url += "&timestamp=" + getTimestamp();
	xmlhttp = ajaxCreateXMLHTTPObject();
	xmlhttp.open("GET", url, true);
	xmlhttp.send(null);
}

//
// Synchronous AJAX call (POST) to URL; return the result from the URL (if any)
//
function ajaxAccessPostUrl(url) {
	// append "?" if the url does not already contain it
	if (url.indexOf("?") == -1) {
		url += "?";
	}
	
	// adding the timestamp to the URL is required in order to access
	// the URL again and not use the cached result
	url += "&timestamp=" + getTimestamp();
	
	// extract the servlet from the URL
	// url = "/context_root/servlet?&a=b&c=d&e=f
	splitUrl = url.split("?");
	servletUrl = splitUrl[0];		// the text before the '?'
	queryString = splitUrl[1];		// the text after the '?'

	// send the request
	xmlhttp = ajaxCreateXMLHTTPObject();
	xmlhttp.open("POST", servletUrl, false);
	xmlhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
	xmlhttp.send(queryString);
	result = trim(xmlhttp.responseText);
	return result;
}

//
// Returns the current timestamp in milliseconds
//
function getTimestamp() {
	now = new Date();
	return now.getTime();
}

//
// Trims leading and trailing spaces from the given value
//
function trim(value) {
	return value.replace(/^\s*|\s*$/g, "");
}

//
// Returns the JSON object created from the jsonText
//
function toJsonObject(jsonText) {
	return eval('(' + jsonText + ')');
}
