/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

//
// Return all locations
//
function ibmctAjaxGetLocations() {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetLocationsAction';
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return a specific location
//
function ibmctAjaxGetLocation(locationId) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetLocationAction' +
		'&portal_location=' + locationId;
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return all EPC pack types defined for a specific EPC profile 
//
function ibmctAjaxGetEpcPackTypes(epcProfileId) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetEpcPackTypesAction' +
		'&epc_profile_id=' + epcProfileId;
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return all assets defined for a specific EPC asset type
//
function ibmctAjaxGetAssetsForEpcAssetType(epcAssetType) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetAssetsForEpcAssetTypeAction' +
		'&epc_asset_type=' + epcAssetType;
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return all assets regardless of EPC pack type defined for a specific EPC profile
//
function ibmctAjaxGetAssetTypes(epcProfileId) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetAssetTypesAction' +
		'&epc_profile=' + epcProfileId;
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return a specific asset
//
function ibmctAjaxGetAsset(assetTypeId) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetAssetAction' +
		'&asset_type=' + assetTypeId;	
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return the cached events
//
function ibmctAjaxGetCachedEvents() {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetEventsAction';
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return all EPC profiles
//
function ibmctAjaxGetEpcProfiles() {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetEpcProfilesAction';
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return all printers
//
function ibmctAjaxGetPrinters() {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetPrintersAction';
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Create an asset
//
function ibmctAjaxCreateAsset(description, epcPackType, epcAssetType, hideAsset, labelDataArray, labelDataKeyAttributeName, labelDataValueAttributeName) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.AdminAction' +
		'&asset_action=create' +
		'&description=' + description +
		'&epc_pack_type=' + epcPackType +
		'&epc_asset_type=' + epcAssetType +
		'&hide_asset=' + hideAsset;
	
	// handle labelDataArray
	for (var i=0; i<labelDataArray.length; i++) {
		url +=
			'&' + labelDataKeyAttributeName + '=' + labelDataArray[i][0] +
			'&' + labelDataValueAttributeName + '=' + labelDataArray[i][1];
	}

	result = ajaxAccessPostUrl(url);
	return toIbmctJsonObject(result);
}

//
// Update an asset
//
function ibmctAjaxUpdateAsset(assetId, description, epcPackType, epcAssetType, hideAsset, labelDataArray, labelDataKeyAttributeName, labelDataValueAttributeName) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.AdminAction' +
		'&asset_action=update' +
		'&asset_id=' + assetId +
		'&description=' + description +
		'&epc_pack_type=' + epcPackType +
		'&epc_asset_type=' + epcAssetType +
		'&hide_asset=' + hideAsset;
	
	// handle labelDataArray
	for (var i=0; i<labelDataArray.length; i++) {
		url +=
			'&' + labelDataKeyAttributeName + '=' + labelDataArray[i][0] +
			'&' + labelDataValueAttributeName + '=' + labelDataArray[i][1];
	}

	result = ajaxAccessPostUrl(url);
	return toIbmctJsonObject(result);
}

//
// Manage a portal
//
function ibmctAjaxPortal(locationId, action, process, supplierId) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.PortalAction' +
		'&portal_location=' + locationId +
		'&portal_action=' + action +
		'&portal_process=' + process +
		'&portal_supplier=' + supplierId;

	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Print a tag
//
function ibmctAjaxPrint(printerId, epcProfileId, assetType, count) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.PrintAction' +
		'&print_printer=' + printerId +
		'&print_epc_profile=' + epcProfileId +
		'&print_asset_type=' + assetType +
		'&print_count=' + count;

	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Reprint a tag
//
function ibmctAjaxReprint(printerId, epcProfileId, tagId) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.ReprintAction' +
		'&reprint_printer=' + printerId +
		'&reprint_epc_profile=' + epcProfileId +
		'&reprint_tagid=' + tagId;

	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return the report's required date format 
//
function ibmctAjaxGetReportDateFormat() {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.ReportAction' +
		'&report_action=get_date_format';
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Build the Observe History report
//
function ibmctAjaxObserveHistoryReport(location, startDate, endDate, reportLocation, messageId, locationId, startDateId, endDateId, submitButtonId) {	
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.ReportAction' +
		'&report_action=build' +
		'&location_id=' + location +
		'&start_date=' + startDate +
		'&end_date=' + endDate;
	
	// adding the timestamp to the URL is required in order to access
	// the URL again and not use the cached result
	url += "&timestamp=" + getTimestamp();

	xmlhttp = ajaxCreateXMLHTTPObject();
	xmlhttp.open("GET", url, true);
 
	xmlhttp.onreadystatechange = function() {
		if (xmlhttp.readyState == 4 && xmlhttp.status == 200) {
			if (xmlhttp.responseText) {
				var jsonObj = toIbmctJsonObject(xmlhttp.responseText);
				
				// any error message has already been displayed by toIbmctJsonObject;
				// just re-enable the form in an error occurred
				if (typeof jsonObj.errorMessage != "undefined") {
					// hide the 'please wait' message
					document.getElementById(messageId).style.display = "inline";
	
					// enable the controls
					enableElement(locationId);
					enableElement(startDateId);
					enableElement(endDateId);
					enableElement(submitButtonId);
				} else {
					// display the report
					sessionId = jsonObj.buildReport.sessionId;
					reportUrl =
						"/birt-viewer/frameset" +
						"?__report=" + encodeURIComponent(reportLocation + 'ObserveHistory/ObserveHistory.rptdesign') +
						"&sessionId=" + encodeURIComponent(sessionId);

					window.location=reportUrl;
				}
			}
		}
	};
	
	xmlhttp.send(null);
}
//
// Return the current report session ID
//
function ibmctAjaxGetSessionId() {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetSessionIdAction';	
	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Return a Premises Server SystemAgent property.  If no defined, return an empty string.
//
function ibmctAjaxGetProperty(key) {
	return ibmctAjaxGetProperty(key, "");
}

//
// Return a Premises Server SystemAgent property.  If not defined, return the default value.
//
function ibmctAjaxGetProperty(key, defaultValue) {
	url = '/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetPropertyAction' +
		'&property=' + key +	
		'&default_value=' + defaultValue;

	result = ajaxAccessUrl(url);
	return toIbmctJsonObject(result);
}

//
// Returns the JSON text converted to an object
//
function toIbmctJsonObject(text) {
	if (text != "") {
		obj = toJsonObject(text);

		if (typeof obj.errorMessage != "undefined") {
			alert(decodeURIComponent(obj.errorMessage));
		}
	
		return obj;
	} else {
		return null;
	}
}
