/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

var wizard = new NotebookWizard();

//
// Construct a new NotebookWizard
//
function NotebookWizard() {
	this.pages = new Array();
	this.pageIndex = 0;
}

//
// Construct a new NotebokWizardPage; each page has a page ID,
// a boolean indicating whether the current user is authorized to
// see the page, and label text
//
function NotebookWizardPage(pageId, isAuthorized, label) {
	this.id = pageId;
	this.isAuthorized = isAuthorized;
	this.label = label;
}

//
// Add a page to the wizard
//
function addPage(pageId, isAuthorized, label) {
	i = wizard.pages.length;
	wizardPage = new NotebookWizardPage(pageId, isAuthorized, label);
	wizard.pages[i] = wizardPage;
	hidePage(i);
//	initWizardPage(pageId);
}

//
// Create a new wizard instance and display the first 
// authorized page
//
function createWizard() {
	try {
		wizard = new NotebookWizard();
		initWizard();
		// display the first authorized page
		for (var i=0; i<wizard.pages.length; i++) {
			wizardPage = getPage(i);
			if (wizardPage.isAuthorized == true) {
				showPage(i);
				break;
			}
		}
	} catch (err) {
		// do nothing
	}
}

//
// Update the wizard tabs with the correct highlighting;
// only authorized pages are displayed
//
function updateWizardTabs() {
	var obj = document.getElementById("NotebookWizardTabs");

	if (obj != null) {
		tabsHTML = '';
		for (var i=0; i<wizard.pages.length; i++) {
			// skip unauthorized pages
			wizardPage = getPage(i);
			if (wizardPage.isAuthorized == false) {
				continue;
			}			
			
			if (i == wizard.pageIndex) {
				tabsHTML += 
					'<TD nowrap class="notebookTabsActive">' +
					'<A class="notebookTabsLink" href="#" onclick="showPage(' + i + '); return false;">' + getPageText(i) + '</A>';
					'</TD>';
			} else {
				tabsHTML +=
					'<TD nowrap class="notebookTabsInactive">' +
					'<A  class="notebookTabsLink" href="#" onclick="showPage(' + i + '); return false;">' + getPageText(i) + '</A>';
					'<TD>';
			}
		}
		
		tabsHTML +=
			'<TD nowrap class="notebookTabsFiller" width="100%">' +
			'&nbsp;' +
			'<TD>';

		obj.innerHTML = 
			'<TABLE>' +
			'<TR>' +
			tabsHTML +
			'</TR>' +
			'<TR><TD><BR></TD></TR>' + 
			'</TABLE>';
	}
}

//
// Returns the label for the given page index
//
function getPageText(i) {
	text = wizard.pages[i].label;
	return text;
}

//
// Returns the page for the given index
//
function getPage(i) {
	return wizard.pages[i];
}

//
// Show the page for the given index
//
function showPage(i) {
	try {
		hidePage(wizard.pageIndex);
		initWizardPage(wizard.pages[i].id);
		document.getElementById(wizard.pages[i].id).style.display = "inline";
		wizard.pageIndex = i;
		updateWizardTabs();
	} catch (err) {
		// do nothing
	}
}

//
// Hide the page for the given index
//
function hidePage(i) {
	try {
		destroyWizardPage(wizard.pages[i].id);
		document.getElementById(wizard.pages[i].id).style.display = "none";
	} catch (err) {
		// do nothing
	}
}

//
// Is the page (by page ID) visible?
//
function isPageVisible(pageId) {
	try {
		isDisplay = document.getElementById(pageId).style.display;
		if (isDisplay == "none") {
			return false;
		} else {
			return true;
		}
	} catch (err) {
		// do nothing
	}
}

//
// Returns the id of the currently selected page
//
function currentPageId() {
	try {
		return wizard.pages[wizard.pageIndex].id;
	} catch (err) {
		return "";
	}
}

