/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

//
// Remove all non-alphanumerics from the value of the DOM element ID;
// must be an element that supports value
//
function removeNonAlphaNumericsFromElementValue(id) {
	var value = getElementValue(id);
	var updatedValue = removeNonAlphaNumerics(value);
	setElementValue(id, updatedValue);
}

//
// Remove all non-alphanumerics from the string s
//
function removeNonAlphaNumerics(s) {
	var t = "";
	var allowedChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890";	// including space
	for (var i=0; i<s.length; i++) {
		if (allowedChars.toLowerCase().indexOf(s.charAt(i).toLowerCase()) != -1) {
			t += s.charAt(i);
		}
	}
	return t;
}

//
// Returns true if fullString contains only the characters in validChars;
// false otherwise
//
function containsOnly(fullString, validChars) {
	if (isEmpty(fullString)) {
		return false;
	}
	
	for (var i=0; i<fullString.length; i++) {
		if (validChars.indexOf(fullString.charAt(i)) == -1) {
			return false;
		}
	}
	return true;
}

//
// Returns true if s is an empty string
//
function isEmpty(s) {
	if (trim(s).length == 0) {
		return true;
	} else {
		return false;
	}
}

//
// Returns string s with v1 substituted for all "{0}"
//
function formatMessage(s, v1) {
	values = new Array(1);
	values[0] = v1;
	return formatMessageValues(s, values);
}

//
// Returns string s with v1 substituted for all "{0}"
// and v2 for all "{1}"
//
function formatMessage(s, v1, v2) {
	values = new Array(2);
	values[0] = v1;
	values[1] = v2;
	return formatMessageValues(s, values);
}

//
// Returns string s with v1 substituted for all "{0}",
// v2 for all "{1}", and v3 for all "{2}"
//
function formatMessage(s, v1, v2, v3) {
	values = new Array(3);
	values[0] = v1;
	values[1] = v2;
	values[2] = v3;
	return formatMessageValues(s, values);
}

//
// Returns string s with elements from array values substituted; 
// for example, substitute values[0] for all "{0}", values[1]
// for all "{1}", etc.
//
function formatMessageValues(s, values) {
	result = s;
	
	for (var i=0; i<values.length; i++) {
		re = new RegExp("\\{" + i + "\\}", "g");
		result = result.replace(re, values[i]);
	}
	
	return result;
}

//
// Returns true if the selectedIndex of the DOM element ID
// is not index; if the selectedIndex is index, display message and
// return false
//
function validateSelectedIndex(id, message, index) {
	elementIndex = getElementIndex(id);
	
	if (elementIndex == index) {
		alert(message);
		setElementFocus(id);
		return false;
	}

	return true;
}

//
// Returns true if the value of the DOM element ID
// is not empty; if the value is empty, display message
// and return false
//
function validateValueNotEmpty(id, message) {
	elementValue = getElementValue(id);
	
	if (isEmpty(elementValue)) {
		alert(message);
		setElementFocus(id);
		return false;
	}

	return true;
}

//
// Returns true if the value of the DOM element ID
// is not numeric -- "0123456789" only; if the value
// is not numeric, display message and return false
//
function validateValueNumeric(id, message) {
	elementValue = getElementValue(id);
	
	if (!containsOnly(elementValue, "0123456789")) {
		alert(message);
		setElementFocus(id);
		return false;
	}

	return true;
}

//
// Returns true if the value of the DOM element ID
// is 0; if the value is 0, display message and return false
//
function validateValueZeroOnly(id, message) {
	elementValue = getElementValue(id);
	
	if (containsOnly(elementValue, "0")) {
		alert(message);
		setElementFocus(id);
		return false;
	}
	
	return true;
}

//
// Returns true if the value of the DOM element ID
// is >= min; if the value is < min, display message and 
// return false
//
function validateValueMin(id, message, min) {
	elementValue = getElementValue(id);
	
	if (elementValue < min) {
		alert(message);
		setElementFocus(id);
		return false;
	}

	return true;
}

//
// Returns true if the value of the DOM element ID
// is <= max; if the value is > max, display message and 
// return false
//
function validateValueMax(id, message, max) {
	elementValue = getElementValue(id);
	
	if (elementValue > max) {
		alert(message);
		setElementFocus(id);
		return false;
	}

	return true;
}

//
// Returns the selectedIndex of the DOM element ID;
// must be an element that supports selectedIndex
//
function getElementIndex(id) {
	return document.getElementById(id).selectedIndex;
}

//
// Sets the selectedIndex of the DOM element ID
// to index; must be an element that supports selectdIndex
//
function setElementIndex(id, index) {
	document.getElementById(id).selectedIndex = index;
}

//
// Returns the checked of the DOM element ID;
// must be an element that supports checked
//
function isElementChecked(id) {
	return document.getElementById(id).checked;
}

//
// Sets the checked of the DOM element ID
// to index; must be an element that supports checked
//
function setElementChecked(id) {
	document.getElementById(id).checked = true;
}

//
// Returns the value of the DOM element ID;
// must be an element that supports value
//
function getElementValue(id) {
	return document.getElementById(id).value;
}

//
// Sets the value of the DOM element ID;
// must be an element that supports value
//
function setElementValue(id, v) {
	document.getElementById(id).value = v;
}

//
// Sets the focus to the DOM element ID;
// must be an element that supports focus()
//
function setElementFocus(id) {
	document.getElementById(id).focus();
}

//
// Enables the DOM element ID
//
function enableElement(id) {
	document.getElementById(id).disabled = false;
}

//
// Disables the DOM element ID
//
function disableElement(id) {
	document.getElementById(id).disabled = true;
}

//
// Trims leading and trailing spaces from the given value
//
function trim(value) {
	return value.replace(/^\s*|\s*$/g, "");
}

//
// Pads the string s with padChar so the result is max characters long
//
function padString(s, max, padChar) {
	var t = s;
	for (var i=0; i<max-s.length; i++) {
		t += padChar;
	}
	return t;
}

//
// Crops the string s at max characters long; if s is longer than
// max characters, crop the string and append the cropChar
// (e.g. cropString('ABCDEGHIJ', 6, '.', 3) returns 'ABC...'
//
function cropString(s, max, cropChar, cropCharCount) {
	var t = s.substr(0, max-cropCharCount);
	for (var i=0; i<cropCharCount; i++) {
		t += cropChar;
	}	
	return t;
}
