/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets;

import java.util.Map;
import java.util.ResourceBundle;

import com.ibm.premises.ct.ejb.RucSessionFacadeLocal;
import com.ibm.premises.ct.ejb.RucSessionFacadeLocalHome;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.PrintRuc;
import com.ibm.premises.ct.servlets.actions.PrintAction;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Thread to handle Print RUC processing
 */
public class PrintThread extends Thread {
	private Utils utils;
	private ResourceBundle rb;
	private RucSessionFacadeLocalHome rucHome;
	private String printerId;
	private String epcProfileId;
	private String epcPackType;
	private int epcAssetType;
	private Map<String, String> labelData;
	private int count;
	
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	protected PrintThread() {
		this.utils = Utils.singleton();
		this.rb = ResourceBundle.getBundle("com.ibm.premises.ct.ui.resources.Messages");
	}
	
	public PrintThread(RucSessionFacadeLocalHome rucHome, String printerId, String epcProfileId, String epcPackType, int epcAssetType, Map<String, String> labelData, int count) {
		this();
		this.rucHome = rucHome;
		this.printerId = printerId;
		this.epcProfileId = epcProfileId;
		this.epcPackType = epcPackType;
		this.epcAssetType = epcAssetType;
		this.labelData = labelData;
		this.count = count;
	}
	
	public void run() {
		Logger logger = this.utils.logger;
		
		try {
			logger.traceExit(this, "run");
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(this.rb.getString("ibmct.ui.trace.info.handler.parameter"), PrintAction.PRINTER, this.printerId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", message);
				
				message = utils.formatMessage(this.rb.getString("ibmct.ui.trace.info.handler.parameter"), PrintAction.EPC_PROFILE, this.epcProfileId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", message);
				
				message = utils.formatMessage(this.rb.getString("ibmct.ui.trace.info.handler.parameter"), "print_epcPackType", this.epcPackType);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", message);
				
				message = utils.formatMessage(this.rb.getString("ibmct.ui.trace.info.handler.parameter"), PrintAction.ASSET_TYPE, this.epcAssetType);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", message);
				
				message = utils.formatMessage(this.rb.getString("ibmct.ui.trace.info.handler.parameter"), "print_labelData", this.labelData.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", message);
				
				message = utils.formatMessage(this.rb.getString("ibmct.ui.trace.info.handler.parameter"), PrintAction.COUNT, String.valueOf(this.count));
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", message);
			}
			
			RucSessionFacadeLocal ruc = this.rucHome.create();
			
			for (int i=0; i<this.count; i++) {
				// call the EPC RUC
				String epcUrn = ruc.epcgen(this.epcProfileId, this.epcPackType, String.valueOf(this.epcAssetType));
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.ruc.epc"), epcUrn);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", message);
				}
			
				if (epcUrn != null) {
					// call the Print RUC
					ruc.print(PrintRuc.MODE_PRINT, this.printerId, this.epcProfileId, this.epcPackType, epcUrn, this.labelData);
				} else {
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "run", rb.getString("ibmct.ui.trace.error.handler.ruc.print.null.epc"));
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "run", e);
		} finally {
			logger.traceExit(this, "run");
		}
	}
	
}
