/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.ResourceBundle;

import javax.naming.InitialContext;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;
import javax.sql.DataSource;

import com.ibm.premises.ct.utils.Constants;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.rfid.premises.logger.Logger;

/**
 * SessionListener tracks when an HttpSession is created/destroyed.  When a session is
 * destroyed, cleanup the CT_REPORT table.
 */
public class SessionListener implements HttpSessionListener {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	protected static Utils utils;
	protected static ResourceBundle rb;
	
	public SessionListener() {
		utils = Utils.singleton();
		rb = ResourceBundle.getBundle("com.ibm.premises.ct.ui.resources.Messages");
	}

	public void sessionCreated(HttpSessionEvent event) {
	}

	public void sessionDestroyed(HttpSessionEvent event) {	
		Logger logger = utils.logger;
		
		Connection con = null;
		PreparedStatement ps = null;
		
		try {
			logger.traceEntry(this, "sessionDestroyed");
			
			// extract the session ID
			String id = (String) event.getSession().getAttribute(Constants.REPORT_SESSION_ID);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.session.listener.session.id"), id);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "sessionDestroyed", message);
			}
			
			if (id != null) {				
				InitialContext ic = new InitialContext();
				DataSource dataSource = (DataSource) ic.lookup(Constants.JDBC_REF);
				
				String sql = "delete from sage.ct_report_observe where session_id=?";
			
				con = dataSource.getConnection();
				ps = con.prepareStatement(sql);	
				ps.setString(1, id);			
				ps.executeUpdate();
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "sessionDestroyed", e);
		} finally {
			try {
				ps.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
			
			logger.traceExit(this, "sessionDestroyed");
		}
	}
}
