/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.premises.ct.servlets.actions.Action;
import com.ibm.premises.ct.servlets.actions.IAction;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Servlet implementation class for Servlet: UIActionHanlder
 *
 */
 public class UIActionHandler extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
		+ "All Rights Reserved * Licensed Materials - Property of IBM "
		+ "US Government Users Restricted Rights - Use, duplication or "
		+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	private static final long serialVersionUID = 1L;
	private static Utils utils;
	private static Map<String, IAction> actionHandlers;
	private static ResourceBundle rb;
	public static final String JSON_TEXT =
		"'{'\"config\": \n" + 
			"{0}\n" +
		"'}'";	
	public static final String JSON_CONFIG_URLS =
		"'{'\"urls\": \n" + 
			"'{'{0}'}'\n" +
		"'}'";
	public static final String JSON_CONFIG_URL =
		"\"{0}\": \"{1}\"";
	
	public UIActionHandler() {
		super();
	}   	

	public void init() throws ServletException {
		super.init();
		utils = Utils.singleton();
		rb = ResourceBundle.getBundle("com.ibm.premises.ct.ui.resources.Messages");
		actionHandlers = new HashMap<String, IAction>();
	}
	
	protected void registerHandler(IAction action) {
		actionHandlers.put(action.getClass().getName(), action);
	}
	
	protected IAction findHandler(String className) throws Exception {
		Logger logger = utils.logger;
		try {
			logger.traceEntry(this, "findHandler");
			
			if (className == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), Action.ACTION_ID);
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findHandler", message);
				}
				throw new ActionException(message);
			}
			
			if (actionHandlers.containsKey(className)) {
				Object handler = actionHandlers.get(className);
				if (handler instanceof IAction) {
					return (IAction) handler;
				} else {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.action.incorrect.type"), className, handler.getClass().getName());
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findHandler", message);
					}
					throw new ActionException(message);
				}
			} else {
				IAction iAction = this.loadHandler(className);
				if (iAction != null) {
					return iAction;
				} else {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.action.not.found"), className);
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "findHandler", message);
					}
				
					throw new ActionException(message);
				}
			}
		} finally {
			logger.traceExit(this, "findHandler");
		}
	}
 
 	protected IAction loadHandler(String className) {
 		try {
			Object obj = Class.forName(className).newInstance();
			if (obj instanceof IAction) {
				this.registerHandler((IAction) obj);
				return (IAction) obj;
			} else {
				return null;
			}
 		} catch (Exception e) {
 			return null;
 		}
 	}
	
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Logger logger = utils.logger;
		try {
			logger.traceEntry(this, "doPost");
			this.doGet(request, response);
		} catch (Exception e) {
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "doPost", e);
			e.printStackTrace();
		} finally {
			logger.traceExit(this, "doPost");
		}
	}  	
	
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "doGet");
			
			if (logger.isTraceEnabled()) {
				boolean hasRequestParms = false;
				for (Enumeration eName=request.getParameterNames(); eName.hasMoreElements(); ) {
					hasRequestParms = true;
					String name = (String) eName.nextElement();
					String value = request.getParameter(name);
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), name, value);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "doGet", message);
				}
				
				if (!hasRequestParms) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "doGet", rb.getString("ibmct.ui.trace.info.handler.no.parameters"));
				}
			}
			
			String actionID = request.getParameter(Action.ACTION_ID);
			String result = null;
			
			IAction action = this.findHandler(actionID);
			result = action.handleAction(request, response);
			if (result != null) {
				this.sendResult(response, result);
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "doGet", e);
			
			String message = utils.escape(rb.getString("ibmct.ui.trace.error"));
			
			// create the JSON text
			String jsonText = utils.formatMessage(Action.JSON_ERROR_MESSAGE, message);
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "doGet", utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText));
			}

			try {
				this.sendResult(response, jsonText);
			} catch (Exception e2) {
				// no-op since this exception has already been logged in sendResult()
			}
		} finally {
			logger.traceExit(this, "doGet");
		}
	}	

	private void sendResult(HttpServletResponse response, String s) throws Exception {
		if (s == null) {
			return;
		}
		
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "sendResult");
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "sendResult", utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.result"), s));
			}
			
			PrintWriter out = response.getWriter();
			response.setContentType("text/plain");
			out.println(s);
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "sendResult", e);
			throw e;
		} finally {
			logger.traceExit(this, "sendResult");
		}
	}
	
}
