/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Admin action
 */
public class AdminAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String UI_OPTIONS_LIST_DIV_ID = "manageAssets_options_list_div";
	public static final String UI_OPTION_ID = "manageAssets_option";
	public static final String UI_OUTPUT_DIV_ID = "manageAssets_output_div";
	public static final String UI_SUBMIT = "manageAssets_submit";

	public static final String UI_TITLE = "manageAssets_title";
	public static final String UI_ASSET_TYPES_DIV = "manageAssets_asset_type_div";
	public static final String UI_EPC_PROFILES_DIV = "manageAssets_epc_profiles_div";
	public static final String UI_EPC_PACK_TYPES_DIV = "manageAssets_epc_pack_types_div";
	
	public static final String UI_LABEL_DATA_LIST = "manageAssets_label_data_list";
	public static final String UI_LABEL_DATA_KEY = "manageAssets_label_data_key";
	public static final String UI_LABEL_DATA_VALUE = "manageAssets_label_data_value";
	public static final String UI_BUTTON_ADD = "manageAssets_add_label_data";
	public static final String UI_BUTTON_DELETE = "manageAssets_delete_label_data";
	public static final String UI_AUTO_ADD_TITLE_PROPERTY = "manageAssets_auto_add_title_property";
	public static final String UI_AUTO_ADD_TITLE_PROPERTY_YES_VALUE = "Y";
	public static final String UI_LABEL_DATA_KEY_TITLE = "title";
	
	public static final String UI_HIDE_ASSET_YES = "true";
	public static final String UI_HIDE_ASSET_NO = "false";
	
	public static final String ACTION_CREATE = "create";
	public static final String ACTION_UPDATE = "update";
	
	public static final String ATTR_ASSET_ACTION = "asset_action";
	public static final String ATTR_ASSET_ID = "asset_id";
	public static final String ATTR_EPC_PROFILE = "epc_profile";
	public static final String ATTR_ASSET_TYPE = "asset_type";
	public static final String ATTR_EPC_PACK_TYPE = "epc_pack_type";
	public static final String ATTR_EPC_ASSET_TYPE = "epc_asset_type";
	public static final String ATTR_DESCRIPTION = "description";
	public static final String ATTR_LABEL_DATA_KEYS = "label_data_keys";
	public static final String ATTR_LABEL_DATA_VALUES = "label_data_values";
	public static final String ATTR_HIDE_ASSET = "hide_asset";
	
	public AdminAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;

		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String assetAction = request.getParameter(ATTR_ASSET_ACTION);
			String description = request.getParameter(ATTR_DESCRIPTION);
			String epcPackType = request.getParameter(ATTR_EPC_PACK_TYPE);
			String epcAssetType = request.getParameter(ATTR_EPC_ASSET_TYPE);
			String hideAsset = request.getParameter(ATTR_HIDE_ASSET);
			String[] labelDataKeys = request.getParameterValues(ATTR_LABEL_DATA_KEYS);
			String[] labelDataValues = request.getParameterValues(ATTR_LABEL_DATA_VALUES);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_ASSET_ACTION, assetAction);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_DESCRIPTION, description);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_EPC_PACK_TYPE, epcPackType);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_EPC_ASSET_TYPE, epcAssetType);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_HIDE_ASSET, hideAsset);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);

				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_LABEL_DATA_KEYS, labelDataKeys != null ? Arrays.asList(labelDataKeys) : "");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_LABEL_DATA_VALUES, labelDataValues != null ? Arrays.asList(labelDataValues) : "");
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (assetAction == null || assetAction.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_ASSET_ACTION);
				throw new ActionException(message);
			}
			
			if (description == null || description.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_DESCRIPTION);
				throw new ActionException(message);
			}
			
			if (epcPackType == null || epcPackType.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_EPC_PACK_TYPE);
				throw new ActionException(message);
			}
			
			if (epcAssetType == null || epcAssetType.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_EPC_ASSET_TYPE);
				throw new ActionException(message);
			}
			
			if (hideAsset == null || hideAsset.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_HIDE_ASSET);
				throw new ActionException(message);
			}
			
			// convert the label data arrays to a map
			Map<String, String> labelData = new HashMap<String, String>();
			if (labelDataKeys != null && labelDataValues != null && labelDataKeys.length == labelDataValues.length) {
				for (int i=0; i<labelDataKeys.length; i++) {
					labelData.put(labelDataKeys[i], labelDataValues[i]);
				}
			}
			
			// are we running in test mode?
			boolean isTest = this.isTestMode();
			
			String jsonMessageText = null;
			
			if (ACTION_CREATE.equals(assetAction)) {
				//
				// create asset
				//
				
				Asset asset = new Asset(this.createUUID());
				asset.setDescription(description);
				asset.setEpcPackType(epcPackType);
				asset.setEpcAssetType(Integer.parseInt(epcAssetType));
				asset.setLabelData(labelData);
				asset.setHidden(UI_HIDE_ASSET_YES.equals(hideAsset));
				
				if (!isTest) {
					JdbcSessionFacadeLocal jdbcEjb = jdbcHome.create();
					jdbcEjb.createAsset(asset);
				}
				
				jsonMessageText = rb.getString("ibmct.ui.admin.create.asset.complete");
				
			} else if (ACTION_UPDATE.equals(assetAction)) {
				//
				// update asset
				//
				
				// query the parameters
				String assetId = request.getParameter(ATTR_ASSET_ID);
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_ASSET_ID, assetId);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				if (assetId == null || assetId.length() == 0) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_ASSET_ID);
					throw new ActionException(message);
				}
				
				if (!isTest) {
					Asset asset = new Asset(assetId);
					asset.setDescription(description);
					asset.setEpcPackType(epcPackType);
					asset.setEpcAssetType(Integer.parseInt(epcAssetType));
					asset.setLabelData(labelData);
					asset.setHidden(UI_HIDE_ASSET_YES.equals(hideAsset));
					
					JdbcSessionFacadeLocal jdbcEjb = jdbcHome.create();
					jdbcEjb.updateAsset(asset);
				}
				
				jsonMessageText = rb.getString("ibmct.ui.admin.edit.assets.complete");
				
			} else {
				//
				// unknown asset action
				//
				
				String message = rb.getString("ibmct.ui.admin.error.unexpected.action");
				throw new ActionException(message);	
			}
			
			// create the JSON text
			String jsonText = this.createJsonStatusMessage(jsonMessageText);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}

			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}

}
