/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get asset action
 */
public class GetAssetAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String ASSET_TYPE = "asset_type";
	
	public static final String JSON_TEXT = 
		"'{'\"assetType\": \n" +
			"{0}\n" +
		"'}'";
	public static final String JSON_ELEMENT =
		"'{'\"id\": \"{0}\", \"description\": \"{1}\", \"epcAssetType\": \"{2}\", \"epcPackType\": \"{3}\", \"hidden\": \"{4}\", \"labelData\": [\n{5}\n]'}'";
	
	public static final String JSON_ELEMENT_LABEL_DATA =
		"'{'\"name\": \"{0}\", \"value\": \"{1}\"'}'";
	
	public GetAssetAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String assetTypeId = request.getParameter(ASSET_TYPE);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ASSET_TYPE, assetTypeId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (assetTypeId == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ASSET_TYPE);
				throw new ActionException(message);
			}
			
			// query the asset type
			JdbcSessionFacadeLocal jdbcEjb = jdbcHome.create();
			Asset asset = jdbcEjb.findAsset(assetTypeId);
			
			// create the JSON text
			StringBuffer jsonLabelData = new StringBuffer();
			Map<String, String> labelData = asset.getLabelData();
			for (Iterator<Map.Entry<String, String>> i=labelData.entrySet().iterator(); i.hasNext(); ) {
				Map.Entry<String, String> me = i.next();
				String name = me.getKey();
				String value = me.getValue();
				
				jsonLabelData.append(utils.formatMessage(JSON_ELEMENT_LABEL_DATA, utils.escape(name), utils.escape(value)));
				
				if (i.hasNext()) {
					jsonLabelData.append(",\n");
				}
			}
			
			String id = asset.getId();
			String description = asset.getDescription();
			int epcAssetType = asset.getEpcAssetType();
			String epcPackType = asset.getEpcPackType();
			boolean hidden = asset.isHidden();
				
			String jsonAssetTypeInfo = utils.formatMessage(
				JSON_ELEMENT,
				new Object[] {
					utils.escape(id), utils.escape(description), utils.escape(String.valueOf(epcAssetType)), 
					utils.escape(epcPackType), utils.escape(String.valueOf(hidden)), jsonLabelData.toString()
				}
			);
			
			String jsonText = utils.formatMessage(JSON_TEXT, jsonAssetTypeInfo);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;

		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
