/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Collection;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get asset for EPC asset type action
 */
public class GetAssetsForEpcAssetTypeAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String EPC_ASSET_TYPE = "epc_asset_type";
	
	public static final String JSON_TEXT = 
		"'{'\"assetTypes\": [\n" +
			"{0}\n" +
		"]'}'";
	public static final String JSON_ELEMENT =
		"'{'\"id\": \"{0}\", \"description\": \"{1}\", \"hidden\": \"{2}\"'}'";
	
	public GetAssetsForEpcAssetTypeAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String epcAssetType = request.getParameter(EPC_ASSET_TYPE);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), EPC_ASSET_TYPE, epcAssetType);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (epcAssetType == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), EPC_ASSET_TYPE);
				throw new ActionException(message);
			}
			
			// query the asset types
			// sort them by description (as defined in the Asset.compareTo method)
			JdbcSessionFacadeLocal jdbcEjb = jdbcHome.create();
			Collection<Asset> assets = jdbcEjb.findAllAssetsForEpcAssetType(Integer.parseInt(epcAssetType));
			
			// create the JSON text
			StringBuffer jsonAssetTypes = new StringBuffer();
			
			for (Iterator<Asset> i=assets.iterator(); i.hasNext(); ) {
				Asset asset = i.next();
				String id = asset.getId();
				String description = asset.getDescription();
				boolean hidden = asset.isHidden();
				
				jsonAssetTypes.append(utils.formatMessage(JSON_ELEMENT, utils.escape(id), utils.escape(description), utils.escape(String.valueOf(hidden))));
				
				if (i.hasNext()) {
					jsonAssetTypes.append(",\n");
				}
			}
			
			String jsonText = utils.formatMessage(JSON_TEXT, jsonAssetTypes.toString());
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;

		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
