/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.ColumnMetaData;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get DB meta data action
 */
public class GetDBMetaDataAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	// url = "/ibmct/UIActionHandler?action=com.ibm.premises.ct.servlets.actions.GetDBMetaDataAction" +
	//			"&db_schema=<DB_SCHEMA>&db_table=<DB_TABLE>";
	
	public static final String DB_SCHEMA = "db_schema";
	public static final String DB_TABLE = "db_table";
	
	public static final String JSON_TEXT = 
		"'{'\"dbMetaData\": [\n" +
			"{0}\n" +
		"]'}'";
	public static final String JSON_ELEMENT =
		"'{'\"name\": \"{0}\", \"dataType\": \"{1}\", \"size\": \"{2}\", \"isNullable\": \"{3}\", \"isPrimaryKey\": \"{4}\"'}'";
	
	public GetDBMetaDataAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String dbSchema = request.getParameter(DB_SCHEMA);
			String dbTable = request.getParameter(DB_TABLE);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), DB_SCHEMA, dbSchema);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), DB_TABLE, dbTable);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}

			if (dbSchema == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), DB_SCHEMA);
				throw new ActionException(message);
			}
			
			if (dbTable == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), DB_TABLE);
				throw new ActionException(message);
			}
			
			// query the DB metadata
			JdbcSessionFacadeLocal jdbcEjb = jdbcHome.create();
			List<ColumnMetaData> columns = jdbcEjb.queryDBMetaData(dbSchema, dbTable);
			
			// create the JSON text
			StringBuffer jsonColumns = new StringBuffer();
			
			for (Iterator<ColumnMetaData> i=columns.iterator(); i.hasNext(); ) {
				ColumnMetaData column = i.next();
				String name = column.getName();
				String dataType = String.valueOf(column.getDataType());
				String size = String.valueOf(column.getSize());
				String isNullable = String.valueOf(column.isNullable());
				String isPrimaryKey = String.valueOf(column.isPrimaryKey());
				
				jsonColumns.append(utils.formatMessage(JSON_ELEMENT, 
					new Object[] {utils.escape(name), utils.escape(dataType), utils.escape(size), utils.escape(isNullable), utils.escape(isPrimaryKey)}));
				
				if (i.hasNext()) {
					jsonColumns.append(",\n");
				}
			}
			
			String jsonText = utils.formatMessage(JSON_TEXT, jsonColumns.toString());
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;

		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
