/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.PremisesProfile;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get EPC pack types action
 */
public class GetEpcPackTypesAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String EPC_PROFILE_ID = "epc_profile_id";
	
	public static final String JSON_TEXT = 
		"'{'\"epcPackTypes\": [\n" +
			"{0}\n" +
		"]'}'";
	public static final String JSON_ELEMENT =
		"'{'\"packType\": \"{0}\"'}'";
	
	public GetEpcPackTypesAction() throws Exception {
		super();
	}
	
	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the EPC profile ID
			String epcProfileId = request.getParameter(EPC_PROFILE_ID);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), EPC_PROFILE_ID, epcProfileId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (epcProfileId == null || epcProfileId.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), EPC_PROFILE_ID);
				throw new ActionException(message);
			}
			
			PremisesSessionFacadeLocal local = premisesHome.create();
			PremisesProfile profile = local.getProfile(epcProfileId);
			
			// create the JSON text
			StringBuffer jsonPackTypes = new StringBuffer();
			
			for (Iterator<String> i=profile.getPackTypeNames().iterator(); i.hasNext(); ) {
				String packType = i.next();
				
				jsonPackTypes.append(utils.formatMessage(JSON_ELEMENT, utils.escape(packType)));
				
				if (i.hasNext()) {
					jsonPackTypes.append(",\n");
				}
			}

			String jsonText = utils.formatMessage(JSON_TEXT, jsonPackTypes.toString());
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
