/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeConstants;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get location action
 */
public class GetLocationAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String LOCATION = "portal_location";
	
	public static final String JSON_TEXT =
		"'{'\"location\": \n" + 
			"'{'\"id\": \"{0}\", \"description\": \"{1}\", \"started\": \"{2}\", \"action\": \"{3}\", \"process\": \"{4}\", \"supplierId\": \"{5}\"'}'\n" +
		"'}'";
	
	
	public GetLocationAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String location = request.getParameter(LOCATION);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), LOCATION, location);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (location == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), LOCATION);
				throw new ActionException(message);
			}
			
			// query the location
			PremisesSessionFacadeLocal local = premisesHome.create();
			PremisesLocation premisesLocation = local.getLocation(location);
			
			String description = premisesLocation.getDescription();
			String started = String.valueOf(premisesLocation.isStarted());
			
			// create the JSON text
			String action = premisesLocation.isStarted() ? PremisesSessionFacadeConstants.ACTION_START : PremisesSessionFacadeConstants.ACTION_STOP;
			String process = local.extractProcessFromEpcisBizStep((String) premisesLocation.getMetaData().get(PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_KEY));

			String supplierId = null;
			if (PremisesSessionFacadeConstants.PROCESS_SHIPPING.equalsIgnoreCase(process)) {
				supplierId = local.extractSupplierFromEpcisShipToBizLocation((String) premisesLocation.getMetaData().get(PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY));
			} else if (PremisesSessionFacadeConstants.PROCESS_RECEIVING.equalsIgnoreCase(process)) {
				supplierId = local.extractSupplierFromEpcisShipFromBizLocation((String) premisesLocation.getMetaData().get(PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY));
			} else {
				supplierId = "";
			}
			
			String jsonText = utils.formatMessage(JSON_TEXT, 
					new Object[] {utils.escape(location), utils.escape(description), utils.escape(started), utils.escape(action), utils.escape(process), utils.escape(supplierId)});
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
