/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Iterator;
import java.util.SortedSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeConstants;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get locations action
 */
public class GetLocationsAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String JSON_TEXT = 
		"'{'\"locations\": [\n" +
			"{0}\n" +
		"]'}'";
	public static final String JSON_ELEMENT =
		"'{'\"id\": \"{0}\", \"description\": \"{1}\", \"started\": \"{2}\", \"action\": \"{3}\", \"process\": \"{4}\", \"supplierId\": \"{5}\"'}'";
	
	public GetLocationsAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the locations
			PremisesSessionFacadeLocal local = premisesHome.create();
			SortedSet<PremisesLocation> locations = local.getAllLocations();
			
			// create the JSON text
			StringBuffer jsonLocations = new StringBuffer();
			
			for (Iterator<PremisesLocation> i=locations.iterator(); i.hasNext(); ) {
				PremisesLocation location = i.next();
				String id = location.getId();
				String description = location.getDescription();
				boolean started = location.isStarted();
				
				String action = location.isStarted() ? PremisesSessionFacadeConstants.ACTION_START : PremisesSessionFacadeConstants.ACTION_STOP;
				String process = local.extractProcessFromEpcisBizStep((String) location.getMetaData().get(PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_KEY));

				String supplierId = null;
				if (PremisesSessionFacadeConstants.PROCESS_SHIPPING.equalsIgnoreCase(process)) {
					supplierId = local.extractSupplierFromEpcisShipToBizLocation((String) location.getMetaData().get(PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY));
				} else if (PremisesSessionFacadeConstants.PROCESS_RECEIVING.equalsIgnoreCase(process)) {
					supplierId = local.extractSupplierFromEpcisShipFromBizLocation((String) location.getMetaData().get(PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY));
				} else {
					supplierId = "";
				}
				
				jsonLocations.append(utils.formatMessage(JSON_ELEMENT, 
						new Object[] {utils.escape(id), utils.escape(description), utils.escape(String.valueOf(started)), utils.escape(action), utils.escape(process), utils.escape(supplierId)}));
				
				if (i.hasNext()) {
					jsonLocations.append(",\n");
				}
			}

			String jsonText = utils.formatMessage(JSON_TEXT, jsonLocations.toString());
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
