/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get property action
 */
public class GetPropertyAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String PROPERTY = "property";
	public static final String DEFAULT_VALUE = "default_value";
	
	public static final String JSON_TEXT =
		"'{'\"property\": \n" + 
			"'{'\"key\": \"{0}\", \"value\": \"{1}\"'}'\n" +
		"'}'";
	
	public GetPropertyAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String property = request.getParameter(PROPERTY);
			String defaultValue = request.getParameter(DEFAULT_VALUE);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), PROPERTY, property);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), DEFAULT_VALUE, defaultValue);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (property == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), PROPERTY);
				throw new ActionException(message);
			}
			
			// query the property value
			String value = null;
			
			if (defaultValue == null || defaultValue.length() == 0) {
				value = this.getPremisesProperty(property);
			} else {
				value = this.getPremisesProperty(property, defaultValue);
			}
			
			// create the JSON text
			String jsonText = utils.formatMessage(JSON_TEXT, utils.escape(property), utils.escape(value));
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
