/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeConstants;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Portal action
 */
public class PortalAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String LOCATION = "portal_location";
	public static final String ACTION = "portal_action";
	public static final String PROCESS = "portal_process";
	public static final String SUPPLIER = "portal_supplier";
	
	public static final String ACTION_START = PremisesSessionFacadeConstants.ACTION_START;
	public static final String ACTION_STOP = PremisesSessionFacadeConstants.ACTION_STOP;
	public static final String PROCESS_SHIPPING = PremisesSessionFacadeConstants.PROCESS_SHIPPING;
	public static final String PROCESS_RECEIVING = PremisesSessionFacadeConstants.PROCESS_RECEIVING;
	
	public static final String UI_LOCATION_OPTIONS = "portal_location_options";
	public static final String UI_PORTAL_NAME = "portal_name";
	public static final String UI_ACTION_OPTIONS = "portal_action_options";
	public static final String UI_PROCESS_OPTIONS = "portal_process_options";
	public static final String UI_SUBMIT = "portal_button";
	
	public PortalAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String location = request.getParameter(LOCATION);
			String action = request.getParameter(ACTION);
			String process = request.getParameter(PROCESS);
			String supplier = request.getParameter(SUPPLIER);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), LOCATION, location);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ACTION, action);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), PROCESS, process);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), SUPPLIER, supplier);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (location == null || location.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), LOCATION);
				throw new ActionException(message);
			}
			
			if (action == null || action.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ACTION);
				throw new ActionException(message);
			}
			
			if (ACTION_START.equals(action)) {
				// start action:  process and supplier are required
				if (process == null || process.length() == 0) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), PROCESS);
					throw new ActionException(message);
				}
				
				if (supplier == null || supplier.length() == 0) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), SUPPLIER);
					throw new ActionException(message);
				}
			}
			
			// are we running in test mode?
			boolean isTest = this.isTestMode();
			
			PremisesSessionFacadeLocal premisesLocal = premisesHome.create();
			
			if (!isTest) {
				// perform the portal action
				if (ACTION_STOP.equals(action)) {
					premisesLocal.stopLocation(location);
				} else if (ACTION_START.equals(action)) {
					premisesLocal.startLocation(location, process, supplier);
				}
			}
			
			// publish the info alert
			PremisesLocation premisesLocation = premisesLocal.getLocation(location);
					
			if (ACTION_STOP.equals(action)) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.portal.action.stop.alert"), premisesLocation.getDescription());
				utils.publishInfoEvent(message);
			} else if (ACTION_START.equals(action) && PROCESS_SHIPPING.equals(process)) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.portal.action.start.shipping.alert"), premisesLocation.getDescription(), supplier);
				utils.publishInfoEvent(message);
			} else if (ACTION_START.equals(action) && PROCESS_RECEIVING.equals(process)) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.portal.action.start.receiving.alert"), premisesLocation.getDescription());
				utils.publishInfoEvent(message);
			} 
			
			// create the JSON text
			String jsonText = this.createJsonStatusMessage(rb.getString("ibmct.ui.portal.complete"));
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
