/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.dto.PremisesPrinter;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.premises.ct.servlets.PrintThread;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Print action
 */
public class PrintAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String PRINTER = "print_printer";
	public static final String EPC_PROFILE = "print_epc_profile";
	public static final String ASSET_TYPE = "print_asset_type";
	public static final String COUNT = "print_count";
	
	public static final String PROPERTY_PRINTER_MAX_COUNT = "com.ibm.premises.ct.printer.max.count";
	public static final String PROPERTY_PRINTER_MAX_COUNT_DEFAULT = "5000";
	
	public static final String UI_PRINTER_OPTIONS = "print_printer_options";
	public static final String UI_EPC_PROFILE_OPTIONS = "print_epc_profile_options";
	public static final String UI_ASSET_TYPE_OPTIONS = "print_asset_type_options";
	public static final String UI_SUBMIT = "print_button";
	
	public PrintAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String printer = request.getParameter(PRINTER);
			String epcProfile = request.getParameter(EPC_PROFILE);
			String assetType = request.getParameter(ASSET_TYPE);
			String sCount = request.getParameter(COUNT);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), PRINTER, printer);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), EPC_PROFILE, epcProfile);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ASSET_TYPE, assetType);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), COUNT, sCount);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (printer == null || printer.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), PRINTER);
				throw new ActionException(message);
			}
			
			if (epcProfile == null || epcProfile.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), EPC_PROFILE);
				throw new ActionException(message);
			}
			
			if (assetType == null || assetType.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ASSET_TYPE);
				throw new ActionException(message);
			}
			
			if (sCount == null || sCount.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), COUNT);
				throw new ActionException(message);
			}
		
			int count = 0;
			try {
				count = Integer.parseInt(sCount);
			} catch (NumberFormatException e) {
				e.printStackTrace();
				logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
				throw new ActionException(e);
			}
		
			// query the asset type info
			Asset asset = this.getAsset(assetType);
			
			// are we running in test mode?
			boolean isTest = this.isTestMode();
			
			if (!isTest) {
				// print the tag(s)
				PrintThread thread = new PrintThread(rucHome, printer, epcProfile, asset.getEpcPackType(), asset.getEpcAssetType(), asset.getLabelData(), count);
				thread.start();
			}
			
			// publish the info alert
			PremisesPrinter premisesPrinter = this.getPrinter(printer);
			String message = utils.formatMessage(rb.getString("ibmct.ui.print.complete.alert"), premisesPrinter.getDescription());
			utils.publishInfoEvent(message);
			
			// create the JSON text
			String jsonText = this.createJsonStatusMessage(rb.getString("ibmct.ui.print.complete"));
			
			if (logger.isTraceEnabled()) {
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
