/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.premises.ct.dto.ObserveReport;
import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeConstants;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocal;
import com.ibm.premises.ct.ejb.RucSessionFacadeLocal;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.ReportingRuc;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.premises.ct.utils.CTLogger;
import com.ibm.premises.ct.utils.Constants;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Report action
 */
public class ReportAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

	public static final String JSON_TEXT_SESSION_ID =
		"'{'\"buildReport\": \n" + 
			"'{'\"sessionId\": \"{0}\"'}'\n" +
		"'}'";
	
	public static final String JSON_TEXT_DATE_FORMAT =
		"'{'\"report\": \n" + 
			"'{'\"dateFormat\": \"{0}\"'}'\n" +
		"'}'";
	
	public static final String UI_REPORT_ID = "report_id";	
	public static final String UI_REPORT_OPTIONS = "report_options";
	public static final String UI_REPORT_OUTPUT = "report_output";
	public static final String UI_SUBMIT = "report_button";
	public static final String UI_SUBMIT_REPORT = "report_submit_button";
	public static final String UI_LOCATION = "report_location";
	public static final String UI_LOCATIONS = "report_locations_list";
	public static final String UI_LOCATION_ALL = "*";
	public static final String UI_START_DATE = "report_start_date";
	public static final String UI_END_DATE = "report_end_date";
	public static final String UI_MESSAGE_ID = "report_message_id";
	
	public static final String ATTR_REPORT_ACTION = "report_action";
	public static final String ATTR_REPORT_ACTION_BUILD = "build";
	public static final String ATTR_REPORT_ACTION_GET_DATE_FORMAT = "get_date_format";
	public static final String ATTR_BIRT_REPORT = "birt_report"; 
	public static final String ATTR_LOCATION = "location_id";
	public static final String ATTR_REPORT_START_DATE = "start_date";
	public static final String ATTR_REPORT_END_DATE = "end_date";
	public static final String ATTR_REPORT_SESSION_ID = "session_id";
	
	public static final String PROP_REPORT_LOCATION = "com.ibm.premises.report.location";
	public static final String PROP_REPORT_LOCATION_DEFAULT = "";
	
	public static final String PROP_CT_REPORT_TAG_ENCODING_FILTER = "com.ibm.premises.ct.report.tag.encoding.filter";
	public static final String PROP_CT_REPORT_TAG_ENCODING_FILTER_ALL = "*";
	public static final String PROP_CT_REPORT_TAG_ENCODING_FILTER_GRAI96 = "grai-96";
	public static final String PROP_CT_REPORT_TAG_ENCODING_FILTER_DEFAULT = PROP_CT_REPORT_TAG_ENCODING_FILTER_ALL;
	
	public static final String PROP_DATE_FORMAT = "com.ibm.rfid.premises.taghistory.search.filter.date.format";
	public static final String PROP_DATE_FORMAT_DEFAULT = "yyyy-MM-dd";
	
	// performance timers
	private long timeBeginBuildReport = 0;
	private long timeEndBuildReport = 0;
	private long timeBeginGetLocationTagHistory = 0;
	private long timeEndGetLocationTagHistory = 0;
	private long totalTimeGetLocationTagHistory = 0;
	private long timeBeginDeleteObserveReport = 0;
	private long timeEndDeleteObserveReport = 0;
	private long timeBeginCreateObserveReportDTO = 0;
	private long timeEndCreateObserveReportDTO = 0;
	private long timeTotalCreateObserveReportDTO = 0;
	private long timeBeginCreateObserveReport = 0;
	private long timeEndCreateObserveReport = 0;
	private long timeTotalCreateObserveReport = 0;
	private long countTotalEventsProcessed = 0;
	private long countTotalReportDTOsCreated = 0;
	
	public ReportAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		CTLogger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			String reportAction = request.getParameter(ATTR_REPORT_ACTION);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_REPORT_ACTION, reportAction);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (reportAction == null || reportAction.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_REPORT_ACTION);
				throw new ActionException(message);
			}
			
			// are we running in test mode?
			boolean isTest = this.isTestMode();	
			
			if (ATTR_REPORT_ACTION_GET_DATE_FORMAT.equals(reportAction)) {
				// 
				// get date format
				//
				
				String dateFormat = this.getDateFormat();
				
				// create the JSON text
				String jsonText = utils.formatMessage(JSON_TEXT_DATE_FORMAT, dateFormat);
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				return jsonText;		
				
			} else if (ATTR_REPORT_ACTION_BUILD.equals(reportAction)) {
				//
				// build report
				//
				
				if (logger.isPerformanceTraceEnabled()) {
					if (ATTR_REPORT_ACTION_BUILD.equals(reportAction)) {
						// reset timers
						timeBeginBuildReport = 0;
						timeEndBuildReport = 0;
						timeBeginGetLocationTagHistory = 0;
						timeEndGetLocationTagHistory = 0;
						totalTimeGetLocationTagHistory = 0;
						timeBeginDeleteObserveReport = 0;
						timeEndDeleteObserveReport = 0;
						timeBeginCreateObserveReportDTO = 0;
						timeEndCreateObserveReportDTO = 0;
						timeTotalCreateObserveReportDTO = 0;
						timeBeginCreateObserveReport = 0;
						timeEndCreateObserveReport = 0;
						timeTotalCreateObserveReport = 0;
						countTotalEventsProcessed = 0;
						countTotalReportDTOsCreated = 0;
						
						timeBeginBuildReport = System.currentTimeMillis();
					}
				}
				
				// query the parameters
				String locationId = request.getParameter(ATTR_LOCATION);
				String startDate = request.getParameter(ATTR_REPORT_START_DATE);
				String endDate = request.getParameter(ATTR_REPORT_END_DATE);
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_LOCATION, locationId);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
					
					message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_REPORT_START_DATE, startDate);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
					
					message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), ATTR_REPORT_END_DATE, endDate);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				if (locationId == null || locationId.length() == 0) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_LOCATION);
					throw new ActionException(message);
				}
				
				if (startDate == null || startDate.length() == 0) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_REPORT_START_DATE);
					throw new ActionException(message);
				}
				
				if (endDate == null || endDate.length() == 0) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), ATTR_REPORT_END_DATE);
					throw new ActionException(message);
				}
				
				String dateFormat = this.getPremisesProperty(PROP_DATE_FORMAT, PROP_DATE_FORMAT_DEFAULT);
				Date dStartDate = utils.convertToDate(startDate, dateFormat, false);
				Date dEndDate = utils.convertToDate(endDate, dateFormat, true);
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.report.start.date"), 
						utils.dateAsString(dStartDate, "yyyy-MM-dd hh:mm:ss a"));
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
					
					message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.report.end.date"), 
						utils.dateAsString(dEndDate, "yyyy-MM-dd hh:mm:ss a"));
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				Map[] result = new Map[0];
				String sessionId = this.getSessionId(request);		
				
				if (!isTest) {
					JdbcSessionFacadeLocal jdbc = jdbcHome.create();
					PremisesSessionFacadeLocal premises = premisesHome.create();
					
					// delete any prior reports					
					if (logger.isPerformanceTraceEnabled()) {
						timeBeginDeleteObserveReport = System.currentTimeMillis();
					}
					
					jdbc.deleteObserveReport(sessionId);
					
					if (logger.isPerformanceTraceEnabled()) {
						timeEndDeleteObserveReport = System.currentTimeMillis();
					}
					
					// query the report
					RucSessionFacadeLocal ruc = rucHome.create();
					
					// determine the set of location IDs to use
					Collection<String> locations = new ArrayList<String>();
					if (UI_LOCATION_ALL.equals(locationId)) {
						Collection<PremisesLocation> premLocations = premises.getAllLocations();
						for (Iterator<PremisesLocation> iPremLocs=premLocations.iterator(); iPremLocs.hasNext(); ) {
							PremisesLocation pl = iPremLocs.next();
							locations.add(pl.getId());
						}
					} else {
						locations.add(locationId);
					}
					
					// loop over the count of days between the start and end dates
					int dayCount = this.daysBetween(dStartDate, dEndDate);
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.report.day.range"), 
							String.valueOf(dayCount), startDate, endDate);
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
					}
					
					Date reportStartDate = dStartDate;
					
					for (int days=0; days<dayCount; days++ ) {
						Date reportEndDate = this.addDate(reportStartDate, 1);
						
						String rucStartDate = utils.formatISO8601Date(reportStartDate);
						String rucEndDate = utils.formatISO8601Date(reportEndDate);
					
						// loop over each location
						for (Iterator<String> iLoc=locations.iterator(); iLoc.hasNext(); ) {
							String id = iLoc.next();
							
							if (logger.isTraceEnabled()) {
								String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.report.location"), id, rucStartDate, rucEndDate);
								logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
							}
							
							if (logger.isPerformanceTraceEnabled()) {
								timeBeginGetLocationTagHistory = 0;
								timeEndGetLocationTagHistory = 0;
								timeBeginCreateObserveReportDTO = 0;
								timeEndCreateObserveReportDTO = 0;
								timeBeginCreateObserveReport = 0;
								timeEndCreateObserveReport = 0;
								
								timeBeginGetLocationTagHistory = System.currentTimeMillis();
							}
							
							// query the location				
							result = ruc.getLocationTagHistory(premises.toEpcisReadPoint(id), rucStartDate, rucEndDate);
						
							if (logger.isPerformanceTraceEnabled()) {
								timeEndGetLocationTagHistory = System.currentTimeMillis();
								totalTimeGetLocationTagHistory += (timeEndGetLocationTagHistory - timeBeginGetLocationTagHistory);
							}
							
							if (logger.isTraceEnabled()) {
								if (result != null) {
									String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.report.count"), String.valueOf(result.length));
									logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
									
									for (int i=0; i<result.length; i++) {
										message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.report.data"), result[i].toString());
										logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
									}
								} else {
									String message = rb.getString("ibmct.ui.trace.info.handler.report.null");
									logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
								}
							}
						
							// process the results
							if (result != null) {
								if (logger.isPerformanceTraceEnabled()) {
									countTotalEventsProcessed += result.length;
								}
								
								String propTagEncodingFilter = this.getPremisesProperty(PROP_CT_REPORT_TAG_ENCODING_FILTER, PROP_CT_REPORT_TAG_ENCODING_FILTER_DEFAULT);
								
								for (int i=0; i<result.length; i++) {
									// create DTO from map
									if (logger.isPerformanceTraceEnabled()) {
										timeBeginCreateObserveReportDTO = System.currentTimeMillis();
									}
									
									List<ObserveReport> observeReports = this.createObserveReportDTO(premises, propTagEncodingFilter, sessionId, result[i]);
									
									if (logger.isPerformanceTraceEnabled()) {
										countTotalReportDTOsCreated += observeReports.size();
										timeEndCreateObserveReportDTO = System.currentTimeMillis();
										timeTotalCreateObserveReportDTO += (timeEndCreateObserveReportDTO - timeBeginCreateObserveReportDTO);
									}
									
									// do not include any empty tag aggregations
									if (observeReports.isEmpty()) {
										continue;
									}
									
									// update database
									if (logger.isPerformanceTraceEnabled()) {
										timeBeginCreateObserveReport = System.currentTimeMillis();
									}
									
									jdbc.createObserveReport(observeReports);
									
									if (logger.isPerformanceTraceEnabled()) {
										timeEndCreateObserveReport = System.currentTimeMillis();
										timeTotalCreateObserveReport += (timeEndCreateObserveReport - timeBeginCreateObserveReport);
									}
								}
							}
						}
						
						// update the start date to prep for the new loop iteration
						reportStartDate = reportEndDate;
					}
				}
				
				if (logger.isPerformanceTraceEnabled()) {
					timeEndBuildReport = System.currentTimeMillis();
					String currentDate = utils.currentDateAsString();

					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", "ReportAction build report performance stats:  " + currentDate);
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> total events returned from RUC: " + countTotalEventsProcessed);
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> total Observe Report DTOs created: " + countTotalReportDTOsCreated);
					
					long timeTotalReport = timeEndBuildReport - timeBeginBuildReport;
					long timeDeleteObserveReport = timeEndDeleteObserveReport - timeBeginDeleteObserveReport;
					
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> totalReport time: " + timeTotalReport + " ms");						
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> deleteObserveReport time: " + timeDeleteObserveReport + " ms (" + utils.percentageAsString(timeDeleteObserveReport, timeTotalReport) + ")");
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> totalTimeGetLocationTagHistory time: " + totalTimeGetLocationTagHistory + " ms (" + utils.percentageAsString(totalTimeGetLocationTagHistory, timeTotalReport) + ")");
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> totalCreateObserveReportDTO time: " + timeTotalCreateObserveReportDTO + " ms (" + utils.percentageAsString(timeTotalCreateObserveReportDTO, timeTotalReport) + ")");
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> totalCreateObserveReport time: " + timeTotalCreateObserveReport + " ms (" + utils.percentageAsString(timeTotalCreateObserveReport, timeTotalReport) + ")");
					logger.trace(CTLogger.TRACE_TYPE_PERF, this, "handleAction", ">>> data in CSV format: '" + currentDate + "','" + 
						countTotalEventsProcessed + "','" + countTotalReportDTOsCreated + "','" +
						timeTotalReport + "','" + timeDeleteObserveReport + "','" + totalTimeGetLocationTagHistory + "','" + 
						timeTotalCreateObserveReportDTO + "','" + timeTotalCreateObserveReport +"'");
				}
				
				// create the JSON text
				String jsonText = utils.formatMessage(JSON_TEXT_SESSION_ID, sessionId);
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				return jsonText;

			} else {
				//
				// unknown report action
				//
				
				String message = rb.getString("ibmct.ui.report.error.unexpected.action");
				throw new ActionException(message);				
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
	/**
	 * Get the report session ID
	 * @param request
	 * @return
	 */
	private String getSessionId(HttpServletRequest request) {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getSessionId");
			
			HttpSession session = request.getSession();
			String sessionId = (String) session.getAttribute(Constants.REPORT_SESSION_ID);
			if (sessionId == null) {
				sessionId = this.createUUID();
				session.setAttribute(Constants.REPORT_SESSION_ID, sessionId);
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), Constants.REPORT_SESSION_ID, sessionId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getSessionId", message);
			}
			
			return sessionId;
		} finally {
			logger.traceExit(this, "getSessionId");
		}
	}
	
	/**
	 * Does the tag encoding filter property include the tagEncoding? 
	 * For example, if the filter is set to "grai96" and the tagEncodig is set to
	 * "grai96", return true.  If the filter is set to "grai96" and the tagEncoding
	 * is set to "sgtin64" return false.  If the filter is set to "*", return true.   
	 * 
	 * @param tagEncodingFilter
	 * @param tagEncoding
	 * @return
	 */
	private boolean includeTagInReport(String tagEncodingFilter, String tagEncoding) {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "includeTagInReport");
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "includeTagInReport", "tagEncodingFilter=" + tagEncodingFilter);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "includeTagInReport", "tagEncoding=" + tagEncoding);
			}
			
			boolean result = false;
			
			if (tagEncodingFilter == null) {
				result = true;
			} else if (tagEncoding == null) {
				result = true;
			} else {
				if (tagEncodingFilter.indexOf(PROP_CT_REPORT_TAG_ENCODING_FILTER_ALL) >= 0) {
					result = true;
				} else {
					result = tagEncodingFilter.toLowerCase().indexOf(tagEncoding.toLowerCase()) >= 0;
				}
			}
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "includeTagInReport", "result=" + result);
			}
			
			return result;
		} finally {
			logger.traceExit(this, "includeTagInReport");
		}
	}

	/**
	 * Create a list of DTO -- one DTO per tag in the map; each DTO is later written to the database
	 * @param premises
	 * @param propTagEncodingFilter
	 * @param sessionId
	 * @param map
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	private List<ObserveReport> createObserveReportDTO(PremisesSessionFacadeLocal premises, String propTagEncodingFilter, String sessionId, Map map) throws Exception {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "createObserveReportDTO");
			
			List<ObserveReport> observeReports = new LinkedList<ObserveReport>();
			
			// handle the tags
			List tagList = new ArrayList();
			if (map.containsKey(ReportingRuc.TAG_LIST)) {
				tagList = new ArrayList((Collection) map.get(ReportingRuc.TAG_LIST));
			}

			if (tagList != null) {
				for (Iterator i=tagList.iterator(); i.hasNext(); ) {
					String tagUrn = (String) i.next();

					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.report.epc.urn"), tagUrn);
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", message);
					}
					
					// do we include this tag in the report?
					boolean includeTag = false;
					if (tagUrn.startsWith(GRAI_96)) {
						if (this.includeTagInReport(propTagEncodingFilter, PROP_CT_REPORT_TAG_ENCODING_FILTER_GRAI96)) {
							includeTag = true;
						}
					} else {
						if (this.includeTagInReport(propTagEncodingFilter, PROP_CT_REPORT_TAG_ENCODING_FILTER_ALL)) {
							includeTag = true;
						}
					}
										
					if (!includeTag) {
						continue;
					}
					
					String tagDescription = "";
					String key = ReportingRuc.EXT_DESCRIPTION + "." + tagUrn.replaceAll(":", "_");
					if (map.containsKey(key)) {
						tagDescription = map.get(key).toString();
					}
					
					String rawUri = "";
					key = ReportingRuc.EXT_TAG_ID + "." + tagUrn.replaceAll(":", "_");
					if (map.containsKey(key)) {
						rawUri = map.get(key).toString();
					}
				
					String bizStep = "";
					if (map.containsKey(ReportingRuc.BIZ_STEP)) {
						bizStep = map.get(ReportingRuc.BIZ_STEP).toString();
					}
				
					String bizLocation = "";
					if (map.containsKey(ReportingRuc.BIZ_LOCATION)) {
						bizLocation = map.get(ReportingRuc.BIZ_LOCATION).toString();
					}
			
					String shipToLocation = "";
					if (map.containsKey(ReportingRuc.SHIP_TO_BIZ_LOCATION)) {
						shipToLocation = map.get(ReportingRuc.SHIP_TO_BIZ_LOCATION).toString();
					}
					
					String shipFromLocation = "";
					if (map.containsKey(ReportingRuc.SHIP_FROM_BIZ_LOCATION)) {
						shipToLocation = map.get(ReportingRuc.SHIP_FROM_BIZ_LOCATION).toString();
					}
				
					// call the Premises facade to extract the data		
					String processAction = "";
					if (bizStep != null) {
						processAction = premises.extractProcessFromEpcisBizStep(bizStep);
					}
				
					String supplierId = null;
					if (PremisesSessionFacadeConstants.PROCESS_SHIPPING.equalsIgnoreCase(processAction)) {
						supplierId = premises.extractSupplierFromEpcisShipToBizLocation(shipToLocation);
					} else if (PremisesSessionFacadeConstants.PROCESS_RECEIVING.equalsIgnoreCase(processAction)) {
						supplierId = premises.extractSupplierFromEpcisShipFromBizLocation(shipFromLocation);
					} else {
						supplierId = "";
					}
				
					String locationId = "";
					if (bizLocation != null) {
						locationId = premises.extractLocationFromEpcisBizLocation(bizLocation);
					}

					// get the location user name
					String locationUserName = locationId;
					if (map.containsKey(ReportingRuc.EXT_LOCATION_USER_NAME)) {
						locationUserName = map.get(ReportingRuc.EXT_LOCATION_USER_NAME).toString();
					}
					
					String recordTime = null;
					if (map.containsKey(ReportingRuc.RECORD_TIME)) {
						recordTime = map.get(ReportingRuc.RECORD_TIME).toString();
					}
				
					String eventTime = null;
					if (map.containsKey(ReportingRuc.EVENT_TIME)) {
						eventTime = map.get(ReportingRuc.EVENT_TIME).toString();
					}
					
					// create the DTO
					String id = this.createUUID();
					ObserveReport dto = new ObserveReport(id);
					dto.setSessionId(sessionId);
					dto.setRecordTime(this.normalizeISO8601DateTime(recordTime));
					dto.setEventTime(this.normalizeISO8601DateTime(eventTime));
					dto.setBizStep(bizStep);
					dto.setProcessAction(processAction);
					dto.setBizLocation(bizLocation);
					dto.setLocationUserName(locationUserName);
					dto.setShipToLocation(shipToLocation);
					dto.setSupplierId(supplierId);
					dto.setTagUrn(tagUrn);
					dto.setTagDescription(tagDescription);
					dto.setTagRawValue(rawUri);
					
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "id=" + dto.getId());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "sessionId=" + dto.getSessionId());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "recordTime=" + dto.getRecordTime());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "eventTime=" + dto.getEventTime());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "bizStep=" + dto.getBizStep());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "processAction=" + dto.getProcessAction());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "bizLocation=" + dto.getBizLocation());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "locationUserName=" + dto.getLocationUserName());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "shipToLocation=" + dto.getShipToLocation());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "supplierId=" + dto.getSupplierId());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "tagUrn=" + dto.getTagUrn());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "tagDescription=" + dto.getTagDescription());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "createObserveReportDTO", "tagRawValue=" + dto.getTagRawValue());
					}
					
					observeReports.add(dto);
				}
			}
	
			return observeReports;
		} catch (Exception e) {
			throw e;
		} finally {
			logger.traceExit(this, "createObserveReportDTO");
		}
	}
	
	/**
	 * Returns the search filter date format string defined in premises.properties; if
	 * the property value isn't a valid java.text.SimpleDateFormat pattern, this method
	 * returns yyyy-MM-dd
	 * @return format string
	 */
	public String getDateFormat() {
		String format = PROP_DATE_FORMAT_DEFAULT;
		try {
			format = this.getPremisesProperty(PROP_DATE_FORMAT);
			// use SimpleDateFormat to validate the date pattern
			new SimpleDateFormat(format);
		} catch (Exception e) {
			format = PROP_DATE_FORMAT_DEFAULT;
		}
		
		return format;
	}
	
	/**
	 * Returns an ISO 8601 formatted date in "yyyy-MM-dd HH:mm:ss" format
	 * @param s
	 * @return
	 */
	private String normalizeISO8601DateTime(String s) {
		// yyyy-MM-ddTHH:mm:ss.MMMZ
		// 012345678901234567890123
		
		String result = s.substring(0, 10) + " " + s.substring(11, 19);
		
		return result;
	}
	
	/**
	 * Returns the number of days between the start and end date
	 * @param start
	 * @param end
	 * @return
	 */
	private int daysBetween(Date start, Date end) {
		long sMillis = start.getTime();
		long eMillis = end.getTime();
			
		// calculate days
		long result = (eMillis - sMillis) / 1000L / 60L / 60L / 24L;
		
		if (result < 0) {
			return 0;
		} else {		
			return (int) result;
		}
	}
	
	/**
	 * Adds the number of days to a date
	 * @param d
	 * @param days
	 * @return
	 */
	private Date addDate(Date d, int days) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(d);
		cal.add(Calendar.DATE, days);
		return cal.getTime();
	}
	
}
